@@ -6,12 +6,10 @@ OPTIONS="
       --enable-faad
       --enable-flac
       --enable-theora
-      --enable-twolame
       --enable-avcodec --enable-merge-ffmpeg
       --enable-dca
       --enable-mpc
       --enable-libass
-      --enable-x264
       --enable-schroedinger
       --enable-realrtsp
       --enable-live555
