@@ -2,7 +2,7 @@
  * fluidsynth.c: Software MIDI synthesizer using libfluidsynth
  *****************************************************************************
  * Copyright © 2007 Rémi Denis-Courmont
- * $Id: bdc8d244ab3ce323cde30b45896b092f5b7c3dff $
+ * $Id: f461f3fd14716f2147a200ba614c53ae6c2712f4 $
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published by
@@ -29,7 +29,7 @@
 #include <vlc_dialog.h>
 
 #include <unistd.h>
-#ifdef _POSIX_VERSION
+#if defined( _POSIX_VERSION ) && !defined(__ANDROID__)
 # include <glob.h>
 #endif
 
@@ -38,11 +38,15 @@
 # define FLUIDSYNTH_NOT_A_DLL
 #endif
 
+#ifndef HAVE_FLUIDLITE_H
 #include <fluidsynth.h>
+#else
+#include <fluidlite.h>
+#endif
 
-#define SOUNDFONT_TEXT N_("Sound fonts")
+#define SOUNDFONT_TEXT N_("SoundFont file")
 #define SOUNDFONT_LONGTEXT N_( \
-    "A sound fonts file is required for software synthesis." )
+    "SoundFont file to use for software synthesis." )
 
 #define CHORUS_TEXT N_("Chorus")
 
@@ -64,7 +68,7 @@ static void Close (vlc_object_t *);
 
 vlc_module_begin ()
     set_description (N_("FluidSynth MIDI synthesizer"))
-    set_capability ("decoder", 100)
+    set_capability ("audio decoder", 100)
     set_shortname (N_("FluidSynth"))
     set_category (CAT_INPUT)
     set_subcategory (SUBCAT_INPUT_ACODEC)
@@ -93,7 +97,7 @@ struct decoder_sys_t
 };
 
 
-static block_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block);
+static int  DecodeBlock (decoder_t *p_dec, block_t *p_block);
 static void Flush (decoder_t *);
 
 static int Open (vlc_object_t *p_this)
@@ -120,7 +124,7 @@ static int Open (vlc_object_t *p_this)
             msg_Err (p_this, "cannot load sound fonts file %s", font_path);
         free (font_path);
     }
-#ifdef _POSIX_VERSION
+#if defined( _POSIX_VERSION ) && !defined(__ANDROID__)
     else
     {
         glob_t gl;
@@ -163,22 +167,19 @@ static int Open (vlc_object_t *p_this)
     fluid_synth_set_reverb_on (p_sys->synth,
                                var_InheritBool (p_this, "synth-reverb"));
 
-    p_dec->fmt_out.i_cat = AUDIO_ES;
     p_dec->fmt_out.audio.i_rate =
         var_InheritInteger (p_this, "synth-sample-rate");;
     fluid_synth_set_sample_rate (p_sys->synth, p_dec->fmt_out.audio.i_rate);
     p_dec->fmt_out.audio.i_channels = 2;
-    p_dec->fmt_out.audio.i_original_channels =
-    p_dec->fmt_out.audio.i_physical_channels =
-        AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT;
+    p_dec->fmt_out.audio.i_physical_channels = AOUT_CHAN_LEFT | AOUT_CHAN_RIGHT;
     p_dec->fmt_out.i_codec = VLC_CODEC_FL32;
     p_dec->fmt_out.audio.i_bitspersample = 32;
     date_Init (&p_sys->end_date, p_dec->fmt_out.audio.i_rate, 1);
     date_Set (&p_sys->end_date, 0);
 
     p_dec->p_sys = p_sys;
-    p_dec->pf_decode_audio = DecodeBlock;
-    p_dec->pf_flush        = Flush;
+    p_dec->pf_decode = DecodeBlock;
+    p_dec->pf_flush  = Flush;
     return VLC_SUCCESS;
 }
 
@@ -205,18 +206,13 @@ static void Flush (decoder_t *p_dec)
             fluid_synth_noteoff (p_sys->synth, channel, note);
 }
 
-static block_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
+static int DecodeBlock (decoder_t *p_dec, block_t *p_block)
 {
-    block_t *p_block;
     decoder_sys_t *p_sys = p_dec->p_sys;
     block_t *p_out = NULL;
 
-    if (pp_block == NULL)
-        return NULL;
-    p_block = *pp_block;
-    if (p_block == NULL)
-        return NULL;
-    *pp_block = NULL;
+    if (p_block == NULL) /* No Drain */
+        return VLCDEC_SUCCESS;
 
     if (p_block->i_flags & (BLOCK_FLAG_DISCONTINUITY|BLOCK_FLAG_CORRUPTED))
     {
@@ -224,7 +220,7 @@ static block_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
         if (p_block->i_flags & BLOCK_FLAG_CORRUPTED)
         {
             block_Release(p_block);
-            return NULL;
+            return VLCDEC_SUCCESS;
         }
     }
 
@@ -293,6 +289,8 @@ static block_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
     if (samples == 0)
         goto drop;
 
+    if (decoder_UpdateAudioFormat (p_dec))
+        goto drop;
     p_out = decoder_NewAudioBuffer (p_dec, samples);
     if (p_out == NULL)
         goto drop;
@@ -304,5 +302,7 @@ static block_t *DecodeBlock (decoder_t *p_dec, block_t **pp_block)
                              p_out->p_buffer, 1, 2);
 drop:
     block_Release (p_block);
-    return p_out;
+    if (p_out != NULL)
+        decoder_QueueAudio (p_dec, p_out);
+    return VLCDEC_SUCCESS;
 }
