@@ -2,7 +2,7 @@
  * parse.c: SPU parser
  *****************************************************************************
  * Copyright (C) 2000-2001, 2005, 2006 VLC authors and VideoLAN
- * $Id: 60ed0dd37edf51e0a6d9989b048dc47fda0dea7b $
+ * $Id: 24eb465a7e8ff518fbc54e6e2420ba33881296db $
  *
  * Authors: Sam Hocevar <sam@zoy.org>
  *          Laurent Aimar <fenrir@via.ecp.fr>
@@ -125,7 +125,7 @@ subpicture_t * ParsePacket( decoder_t *p_dec )
      *  one byte gaves two nibbles and may be used twice (once per field)
      * generating 4 codes.
      */
-    spu_data.p_data = malloc( sizeof(*spu_data.p_data) * 2 * 2 * p_sys->i_rle_size );
+    spu_data.p_data = vlc_alloc( p_sys->i_rle_size, sizeof(*spu_data.p_data) * 2 * 2 );
 
     /* We try to display it */
     if( ParseRLE( p_dec, &spu_data, &spu_properties ) )
@@ -238,6 +238,10 @@ static int ParseControlSeq( decoder_t *p_dec, subpicture_t *p_spu,
         case SPU_CMD_FORCE_DISPLAY: /* 00 (force displaying) */
             p_spu->i_start = i_pts + date;
             p_spu->b_ephemer = true;
+            /* ignores picture date as display start time
+             * works around non displayable (offset by few ms)
+             * spu menu over still frame in SPU_Select */
+            p_spu->b_subtitle = false;
             i_index += 1;
             break;
 
@@ -260,7 +264,7 @@ static int ParseControlSeq( decoder_t *p_dec, subpicture_t *p_spu,
                 return VLC_EGENERIC;
             }
 
-            if( p_dec->fmt_in.subs.spu.palette[0] == 0xBeeF )
+            if( p_dec->fmt_in.subs.spu.palette[0] == SPU_PALETTE_DEFINED )
             {
                 unsigned int idx[4];
                 int i;
@@ -322,6 +326,11 @@ static int ParseControlSeq( decoder_t *p_dec, subpicture_t *p_spu,
             p_spu_properties->i_height = (((p_sys->buffer[i_index+5]&0x0f)<<8)|
                               p_sys->buffer[i_index+6]) - p_spu_properties->i_y + 1;
 
+            if (p_spu_properties->i_width < 0 || p_spu_properties->i_height < 0) {
+                msg_Err( p_dec, "integer overflow in SPU command" );
+                return VLC_EGENERIC;
+            }
+
             /* Auto crop fullscreen subtitles */
             if( p_spu_properties->i_height > 250 )
                 p_spu_data->b_auto_crop = true;
@@ -692,8 +701,7 @@ static void Render( decoder_t *p_dec, subpicture_t *p_spu,
     video_palette_t palette;
 
     /* Create a new subpicture region */
-    memset( &fmt, 0, sizeof(video_format_t) );
-    fmt.i_chroma = VLC_CODEC_YUVP;
+    video_format_Init( &fmt, VLC_CODEC_YUVP );
     fmt.i_sar_num = 0; /* 0 means use aspect ratio of background video */
     fmt.i_sar_den = 1;
     fmt.i_width = fmt.i_visible_width = p_spu_properties->i_width;
@@ -713,6 +721,8 @@ static void Render( decoder_t *p_dec, subpicture_t *p_spu,
     p_spu->p_region = subpicture_region_New( &fmt );
     if( !p_spu->p_region )
     {
+        fmt.p_palette = NULL;
+        video_format_Clean( &fmt );
         msg_Err( p_dec, "cannot allocate SPU region" );
         return;
     }
@@ -734,4 +744,7 @@ static void Render( decoder_t *p_dec, subpicture_t *p_spu,
             memset( p_p + i_x + i_y, i_color, i_len );
         }
     }
+
+    fmt.p_palette = NULL;
+    video_format_Clean( &fmt );
 }
