@@ -5,7 +5,7 @@
  * Copyright © 2007-2012 Mirsal Ennaime
  * Copyright © 2009-2012 The VideoLAN team
  * Copyright © 2013      Alex Merry
- * $Id: 94241260232301f7fb695e97bdb0de9610349db3 $
+ * $Id: 77923daa02753ab6427c96d744e17d3f96003c32 $
  *
  * Authors:    Rafaël Carré <funman at videolanorg>
  *             Mirsal Ennaime <mirsal at mirsal fr>
@@ -75,6 +75,7 @@
 #define DBUS_INSTANCE_ID_PREFIX "instance"
 
 #define SEEK_THRESHOLD 1000 /* µsec */
+#define EVENTS_DELAY INT64_C(100000) /* 100 ms */
 
 /*****************************************************************************
  * Local prototypes.
@@ -91,8 +92,6 @@ static const DBusObjectPathVTable dbus_mpris_vtable = {
 typedef struct
 {
     int signal;
-    int i_node;
-    int i_item;
 } callback_info_t;
 
 enum
@@ -195,13 +194,11 @@ static int Open( vlc_object_t *p_this )
             &dbus_mpris_vtable, p_this );
 
     /* Try to register org.mpris.MediaPlayer2.vlc */
-    dbus_bus_request_name( p_conn, DBUS_MPRIS_BUS_NAME, 0, &error );
-    if( dbus_error_is_set( &error ) )
+    const unsigned bus_flags = DBUS_NAME_FLAG_DO_NOT_QUEUE;
+    var_Create(p_intf->obj.libvlc, "dbus-mpris-name", VLC_VAR_STRING);
+    if( dbus_bus_request_name( p_conn, DBUS_MPRIS_BUS_NAME, bus_flags, NULL )
+                                     != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER )
     {
-        msg_Dbg( p_this, "Failed to get service name %s: %s",
-                 DBUS_MPRIS_BUS_NAME, error.message );
-        dbus_error_free( &error );
-
         /* Register an instance-specific well known name of the form
          * org.mpris.MediaPlayer2.vlc.instanceXXXX where XXXX is the
          * current Process ID */
@@ -212,33 +209,33 @@ static int Open( vlc_object_t *p_this )
                   DBUS_MPRIS_BUS_NAME"."DBUS_INSTANCE_ID_PREFIX"%"PRIu32,
                   (uint32_t)getpid() );
 
-        dbus_bus_request_name( p_conn, unique_service, 0, &error );
-        if( dbus_error_is_set( &error ) )
+        if( dbus_bus_request_name( p_conn, unique_service, bus_flags, NULL )
+                                     == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER )
         {
-            msg_Err( p_this, "Failed to get service name %s: %s",
-                     DBUS_MPRIS_BUS_NAME, error.message );
-            dbus_error_free( &error );
-        }
-        else
             msg_Dbg( p_intf, "listening on dbus as: %s", unique_service );
+            var_SetString(p_intf->obj.libvlc, "dbus-mpris-name",
+                          unique_service);
+        }
     }
     else
+    {
         msg_Dbg( p_intf, "listening on dbus as: %s", DBUS_MPRIS_BUS_NAME );
-
+        var_SetString(p_intf->obj.libvlc, "dbus-mpris-name",
+                      DBUS_MPRIS_BUS_NAME);
+    }
     dbus_connection_flush( p_conn );
 
     p_intf->p_sys = p_sys;
     p_sys->p_conn = p_conn;
-    p_sys->p_events = vlc_array_new();
-    p_sys->p_timeouts = vlc_array_new();
-    p_sys->p_watches = vlc_array_new();
+    vlc_array_init( &p_sys->events );
+    vlc_array_init( &p_sys->timeouts );
+    vlc_array_init( &p_sys->watches );
     vlc_mutex_init( &p_sys->lock );
 
     p_playlist = pl_Get( p_intf );
     p_sys->p_playlist = p_playlist;
 
     var_AddCallback( p_playlist, "input-current", AllCallback, p_intf );
-    var_AddCallback( p_playlist, "intf-change", AllCallback, p_intf );
     var_AddCallback( p_playlist, "volume", AllCallback, p_intf );
     var_AddCallback( p_playlist, "mute", AllCallback, p_intf );
     var_AddCallback( p_playlist, "playlist-item-append", AllCallback, p_intf );
@@ -268,16 +265,13 @@ static int Open( vlc_object_t *p_this )
     return VLC_SUCCESS;
 
 error:
+    var_Destroy(p_intf->obj.libvlc, "dbus-mpris-name");
     /* The dbus connection is private,
      * so we are responsible for closing it
      * XXX: Does this make sense when OOM ? */
     dbus_connection_close( p_sys->p_conn );
     dbus_connection_unref( p_conn );
 
-    vlc_array_destroy( p_sys->p_events );
-    vlc_array_destroy( p_sys->p_timeouts );
-    vlc_array_destroy( p_sys->p_watches );
-
     vlc_mutex_destroy( &p_sys->lock );
 
     vlc_close( p_sys->p_pipe_fds[1] );
@@ -300,7 +294,6 @@ static void Close   ( vlc_object_t *p_this )
     vlc_join( p_sys->thread, NULL );
 
     var_DelCallback( p_playlist, "input-current", AllCallback, p_intf );
-    var_DelCallback( p_playlist, "intf-change", AllCallback, p_intf );
     var_DelCallback( p_playlist, "volume", AllCallback, p_intf );
     var_DelCallback( p_playlist, "mute", AllCallback, p_intf );
     var_DelCallback( p_playlist, "playlist-item-append", AllCallback, p_intf );
@@ -324,15 +317,15 @@ static void Close   ( vlc_object_t *p_this )
     dbus_connection_unref( p_sys->p_conn );
 
     // Free the events array
-    for( int i = 0; i < vlc_array_count( p_sys->p_events ); i++ )
+    for( size_t i = 0; i < vlc_array_count( &p_sys->events ); i++ )
     {
-        callback_info_t* info = vlc_array_item_at_index( p_sys->p_events, i );
+        callback_info_t* info = vlc_array_item_at_index( &p_sys->events, i );
         free( info );
     }
     vlc_mutex_destroy( &p_sys->lock );
-    vlc_array_destroy( p_sys->p_events );
-    vlc_array_destroy( p_sys->p_timeouts );
-    vlc_array_destroy( p_sys->p_watches );
+    vlc_array_clear( &p_sys->events );
+    vlc_array_clear( &p_sys->timeouts );
+    vlc_array_clear( &p_sys->watches );
     vlc_close( p_sys->p_pipe_fds[1] );
     vlc_close( p_sys->p_pipe_fds[0] );
     free( p_sys );
@@ -350,7 +343,7 @@ static dbus_bool_t add_timeout(DBusTimeout *to, void *data)
     dbus_timeout_set_data(to, expiry, free);
 
     vlc_mutex_lock(&sys->lock);
-    vlc_array_append(sys->p_timeouts, to);
+    vlc_array_append_or_abort(&sys->timeouts, to);
     vlc_mutex_unlock(&sys->lock);
 
     return TRUE;
@@ -360,10 +353,11 @@ static void remove_timeout(DBusTimeout *to, void *data)
 {
     intf_thread_t *intf = data;
     intf_sys_t *sys = intf->p_sys;
-    unsigned idx = vlc_array_index_of_item(sys->p_timeouts, to);
+    size_t idx;
 
     vlc_mutex_lock(&sys->lock);
-    vlc_array_remove(sys->p_timeouts, idx);
+    idx = vlc_array_index_of_item(&sys->timeouts, to);
+    vlc_array_remove(&sys->timeouts, idx);
     vlc_mutex_unlock(&sys->lock);
 }
 
@@ -391,11 +385,11 @@ static int next_timeout(intf_thread_t *intf)
 {
     intf_sys_t *sys = intf->p_sys;
     mtime_t next_timeout = LAST_MDATE;
-    unsigned count = vlc_array_count(sys->p_timeouts);
+    unsigned count = vlc_array_count(&sys->timeouts);
 
     for (unsigned i = 0; i < count; i++)
     {
-        DBusTimeout *to = vlc_array_item_at_index(sys->p_timeouts, i);
+        DBusTimeout *to = vlc_array_item_at_index(&sys->timeouts, i);
 
         if (!dbus_timeout_get_enabled(to))
             continue;
@@ -426,9 +420,9 @@ static void process_timeouts(intf_thread_t *intf)
 {
     intf_sys_t *sys = intf->p_sys;
 
-    for (int i = 0; i < vlc_array_count(sys->p_timeouts); i++)
+    for (size_t i = 0; i < vlc_array_count(&sys->timeouts); i++)
     {
-        DBusTimeout *to = vlc_array_item_at_index(sys->p_timeouts, i);
+        DBusTimeout *to = vlc_array_item_at_index(&sys->timeouts, i);
 
         if (!dbus_timeout_get_enabled(to))
             continue;
@@ -454,7 +448,7 @@ static dbus_bool_t add_watch( DBusWatch *p_watch, void *p_data )
     intf_sys_t    *p_sys  = (intf_sys_t*) p_intf->p_sys;
 
     vlc_mutex_lock( &p_sys->lock );
-    vlc_array_append( p_sys->p_watches, p_watch );
+    vlc_array_append_or_abort( &p_sys->watches, p_watch );
     vlc_mutex_unlock( &p_sys->lock );
 
     return TRUE;
@@ -464,12 +458,11 @@ static void remove_watch( DBusWatch *p_watch, void *p_data )
 {
     intf_thread_t *p_intf = (intf_thread_t*) p_data;
     intf_sys_t    *p_sys  = (intf_sys_t*) p_intf->p_sys;
+    size_t idx;
 
     vlc_mutex_lock( &p_sys->lock );
-
-    vlc_array_remove( p_sys->p_watches,
-                      vlc_array_index_of_item( p_sys->p_watches, p_watch ) );
-
+    idx = vlc_array_index_of_item( &p_sys->watches, p_watch );
+    vlc_array_remove( &p_sys->watches, idx );
     vlc_mutex_unlock( &p_sys->lock );
 }
 
@@ -497,15 +490,16 @@ static void watch_toggled( DBusWatch *p_watch, void *p_data )
 static int GetPollFds( intf_thread_t *p_intf, struct pollfd *p_fds )
 {
     intf_sys_t *p_sys = p_intf->p_sys;
-    int i_fds = 1, i_watches = vlc_array_count( p_sys->p_watches );
+    size_t i_watches = vlc_array_count( &p_sys->watches );
+    int i_fds = 1;
 
     p_fds[0].fd = p_sys->p_pipe_fds[PIPE_OUT];
     p_fds[0].events = POLLIN | POLLPRI;
 
-    for( int i = 0; i < i_watches; i++ )
+    for( size_t i = 0; i < i_watches; i++ )
     {
         DBusWatch *p_watch = NULL;
-        p_watch = vlc_array_item_at_index( p_sys->p_watches, i );
+        p_watch = vlc_array_item_at_index( &p_sys->watches, i );
         if( !dbus_watch_get_enabled( p_watch ) )
             continue;
 
@@ -555,7 +549,6 @@ static void ProcessEvents( intf_thread_t *p_intf,
 
             vlc_dictionary_insert( &player_properties, "Metadata", NULL );
             break;
-        case SIGNAL_INTF_CHANGE:
         case SIGNAL_PLAYLIST_ITEM_APPEND:
         case SIGNAL_PLAYLIST_ITEM_DELETED:
         {
@@ -616,33 +609,21 @@ static void ProcessEvents( intf_thread_t *p_intf,
             vlc_dictionary_insert( &player_properties, "CanPause", NULL );
             break;
         case SIGNAL_SEEK:
-        {
-            input_thread_t *p_input;
-            input_item_t *p_item;
-            p_input = pl_CurrentInput( p_intf );
-            if( p_input )
-            {
-                p_item = input_GetItem( p_input );
-                vlc_object_release( p_input );
-
-                if( p_item && ( p_item->i_id == p_events[i]->i_item ) )
-                    SeekedEmit( p_intf );
-            }
+            SeekedEmit( p_intf );
             break;
-        }
         default:
             vlc_assert_unreachable();
         }
         free( p_events[i] );
     }
 
-    if( vlc_dictionary_keys_count( &player_properties ) )
+    if( !vlc_dictionary_is_empty( &player_properties ) )
         PlayerPropertiesChangedEmit( p_intf, &player_properties );
 
-    if( vlc_dictionary_keys_count( &tracklist_properties ) )
+    if( !vlc_dictionary_is_empty( &tracklist_properties ) )
         TrackListPropertiesChangedEmit( p_intf, &tracklist_properties );
 
-    if( vlc_dictionary_keys_count( &root_properties ) )
+    if( !vlc_dictionary_is_empty( &root_properties ) )
         RootPropertiesChangedEmit( p_intf, &root_properties );
 
     vlc_dictionary_clear( &player_properties,    NULL, NULL );
@@ -788,11 +769,13 @@ static void *Run( void *data )
 
     int canc = vlc_savecancel();
 
+    mtime_t events_last_date = VLC_TS_INVALID;
+    int events_poll_timeout = -1;
     for( ;; )
     {
         vlc_mutex_lock( &p_sys->lock );
 
-        int i_watches = vlc_array_count( p_sys->p_watches );
+        size_t i_watches = vlc_array_count( &p_sys->watches );
         struct pollfd fds[i_watches];
         memset(fds, 0, sizeof fds);
 
@@ -803,6 +786,8 @@ static void *Run( void *data )
 
         /* thread cancellation is allowed while the main loop sleeps */
         vlc_restorecancel( canc );
+        if( timeout == -1 )
+            timeout = events_poll_timeout;
 
         while (poll(fds, i_fds, timeout) == -1)
         {
@@ -833,26 +818,52 @@ static void *Run( void *data )
         process_timeouts(p_intf);
 
         /* Get the list of watches to process */
-        i_watches = vlc_array_count( p_sys->p_watches );
+        i_watches = vlc_array_count( &p_sys->watches );
         DBusWatch *p_watches[i_watches ? i_watches : 1];
-        for( int i = 0; i < i_watches; i++ )
+        for( size_t i = 0; i < i_watches; i++ )
         {
-            p_watches[i] = vlc_array_item_at_index( p_sys->p_watches, i );
+            p_watches[i] = vlc_array_item_at_index( &p_sys->watches, i );
         }
 
         /* Get the list of events to process */
-        int i_events = vlc_array_count( p_intf->p_sys->p_events );
-        callback_info_t* p_info[i_events ? i_events : 1];
-        for( int i = i_events - 1; i >= 0; i-- )
+        size_t i_events = vlc_array_count( &p_sys->events );
+        callback_info_t** pp_info = NULL;
+
+        if( i_events > 0 )
         {
-            p_info[i] = vlc_array_item_at_index( p_intf->p_sys->p_events, i );
-            vlc_array_remove( p_intf->p_sys->p_events, i );
+            mtime_t now = mdate();
+            if( now - events_last_date > EVENTS_DELAY )
+            {
+                /* Send events every EVENTS_DELAY */
+                events_last_date = now;
+                events_poll_timeout = -1;
+
+                pp_info = vlc_alloc( i_events, sizeof(*pp_info) );
+                if( pp_info )
+                {
+                    for( size_t i = 0; i < i_events; i++ )
+                        pp_info[i] = vlc_array_item_at_index( &p_sys->events, i );
+                    vlc_array_clear( &p_sys->events );
+                }
+            }
+            else if( events_poll_timeout == -1 )
+            {
+                /* Request poll to wake up in order to send these events after
+                 * some delay */
+                events_poll_timeout = ( EVENTS_DELAY - ( now - events_last_date ) ) / 1000;
+            }
         }
+        else /* No events: clear timeout */
+            events_poll_timeout = -1;
 
         /* now we can release the lock and process what's pending */
         vlc_mutex_unlock( &p_intf->p_sys->lock );
 
-        ProcessEvents( p_intf, p_info, i_events );
+        if( pp_info )
+        {
+            ProcessEvents( p_intf, pp_info, i_events );
+            free( pp_info );
+        }
         ProcessWatches( p_intf, p_watches, i_watches, fds, i_fds );
 
         DispatchDBusMessages( p_intf );
@@ -871,6 +882,29 @@ static void   wakeup_main_loop( void *p_data )
                  vlc_strerror_c(errno) );
 }
 
+static bool add_event_locked( intf_thread_t *p_intf, callback_info_t *p_info )
+{
+    if( !p_info->signal )
+    {
+        free( p_info );
+        return false;
+    }
+
+    for( size_t i = 0; i < vlc_array_count( &p_intf->p_sys->events ); ++ i )
+    {
+        callback_info_t *oldinfo =
+            vlc_array_item_at_index( &p_intf->p_sys->events, i );
+        if( p_info->signal == oldinfo->signal )
+        {
+            free( p_info );
+            return false;
+        }
+    }
+
+    vlc_array_append( &p_intf->p_sys->events, p_info );
+    return true;
+}
+
 /* Flls a callback_info_t data structure in response
  * to an "intf-event" input event.
  *
@@ -946,7 +980,6 @@ static int InputCallback( vlc_object_t *p_this, const char *psz_var,
                 break;
 
             p_info->signal = SIGNAL_SEEK;
-            p_info->i_item = input_GetItem( p_input )->i_id;
             break;
         }
         default:
@@ -961,13 +994,11 @@ static int InputCallback( vlc_object_t *p_this, const char *psz_var,
         p_sys->i_playing_state = i_state;
         p_info->signal = SIGNAL_STATE;
     }
-    if( p_info->signal )
-        vlc_array_append( p_intf->p_sys->p_events, p_info );
-    else
-        free( p_info );
+    bool added = add_event_locked( p_intf, p_info );
     vlc_mutex_unlock( &p_intf->p_sys->lock );
 
-    wakeup_main_loop( p_intf );
+    if( added )
+        wakeup_main_loop( p_intf );
 
     (void)psz_var;
     (void)oldval;
@@ -994,13 +1025,8 @@ static int AllCallback( vlc_object_t *p_this, const char *psz_var,
         if( oldval.b_bool != newval.b_bool )
             info.signal = SIGNAL_VOLUME_MUTED;
     }
-    else if( !strcmp( "intf-change", psz_var ) )
-        info.signal = SIGNAL_INTF_CHANGE;
     else if( !strcmp( "playlist-item-append", psz_var ) )
-    {
         info.signal = SIGNAL_PLAYLIST_ITEM_APPEND;
-        info.i_node = ((playlist_add_t*)newval.p_address)->i_node;
-    }
     else if( !strcmp( "playlist-item-deleted", psz_var ) )
         info.signal = SIGNAL_PLAYLIST_ITEM_DELETED;
     else if( !strcmp( "random", psz_var ) )
@@ -1028,10 +1054,12 @@ static int AllCallback( vlc_object_t *p_this, const char *psz_var,
     // Append the event
     *p_info = info;
     vlc_mutex_lock( &p_intf->p_sys->lock );
-    vlc_array_append( p_intf->p_sys->p_events, p_info );
+    bool added = add_event_locked( p_intf, p_info );
     vlc_mutex_unlock( &p_intf->p_sys->lock );
 
-    wakeup_main_loop( p_intf );
+    if( added )
+        wakeup_main_loop( p_intf );
+
     (void) p_this;
     return VLC_SUCCESS;
 }
@@ -1160,9 +1188,9 @@ int DemarshalSetPropertyValue( DBusMessage *p_msg, void *p_arg )
         free( psz ); \
     }
 
-int GetInputMeta( input_item_t* p_input,
-                  DBusMessageIter *args )
+int GetInputMeta( playlist_item_t *item, DBusMessageIter *args )
 {
+    input_item_t *p_input = item->p_input;
     DBusMessageIter dict, dict_entry, variant, list;
     /** The duration of the track can be expressed in second, milli-seconds and
         µ-seconds */
@@ -1171,18 +1199,19 @@ int GetInputMeta( input_item_t* p_input,
     dbus_int64_t i_length = i_mtime / 1000;
     char *psz_trackid;
 
-    if( -1 == asprintf( &psz_trackid, MPRIS_TRACKID_FORMAT, p_input->i_id ) )
+    if( -1 == asprintf( &psz_trackid, MPRIS_TRACKID_FORMAT, item->i_id ) )
         return VLC_ENOMEM;
 
     const char* ppsz_meta_items[] =
     {
-    "mpris:trackid", "xesam:url", "xesam:title", "xesam:artist", "xesam:album",
-    "xesam:tracknumber", "vlc:time", "mpris:length", "xesam:genre",
-    "xesam:userRating", "xesam:contentCreated", "mpris:artUrl", "mb:trackId",
-    "vlc:audio-bitrate", "vlc:audio-samplerate", "vlc:video-bitrate",
-    "vlc:audio-codec", "vlc:copyright", "xesam:comment", "vlc:encodedby",
-    "language", "vlc:length", "vlc:nowplaying", "vlc:publisher", "vlc:setting",
-    "status", "vlc:url", "vlc:video-codec"
+        "mpris:trackid", "xesam:url", "xesam:title", "xesam:artist",
+        "xesam:album", "xesam:tracknumber", "vlc:time", "mpris:length",
+        "xesam:genre", "xesam:userRating", "xesam:contentCreated",
+        "mpris:artUrl", "mb:trackId", "vlc:audio-bitrate",
+        "vlc:audio-samplerate", "vlc:video-bitrate", "vlc:audio-codec",
+        "vlc:copyright", "xesam:comment", "vlc:encodedby", "language",
+        "vlc:length", "vlc:nowplaying", "vlc:publisher", "vlc:setting",
+        "status", "vlc:url", "vlc:video-codec"
     };
 
     dbus_message_iter_open_container( args, DBUS_TYPE_ARRAY, "{sv}", &dict );
