@@ -27,16 +27,26 @@
 
 #include "AbstractAdaptationLogic.h"
 
+#include <limits>
+
 using namespace adaptive::logic;
 
 AbstractAdaptationLogic::AbstractAdaptationLogic    ()
 {
+    maxwidth = std::numeric_limits<int>::max();
+    maxheight = std::numeric_limits<int>::max();
 }
 
 AbstractAdaptationLogic::~AbstractAdaptationLogic   ()
 {
 }
 
-void AbstractAdaptationLogic::updateDownloadRate    (size_t, mtime_t)
+void AbstractAdaptationLogic::updateDownloadRate    (const adaptive::ID &, size_t, mtime_t)
 {
 }
+
+void AbstractAdaptationLogic::setMaxDeviceResolution (int w, int h)
+{
+    maxwidth = (w > 0) ? w : std::numeric_limits<int>::max();
+    maxheight = (h > 0) ? h : std::numeric_limits<int>::max();
+}
