@@ -32,6 +32,7 @@
 
 #include <vlc_common.h>
 #include <vlc_arrays.h>
+#include <cassert>
 
 using namespace adaptive::playlist;
 
@@ -63,12 +64,17 @@ void BasePeriod::addAdaptationSet(BaseAdaptationSet *adaptationSet)
 {
     if ( adaptationSet != NULL )
     {
+        if(adaptationSet->getRepresentations().empty())
+        {
+            assert(!adaptationSet->getRepresentations().empty());
+            return; /* will leak */
+        }
         adaptationSets.push_back(adaptationSet);
         childs.push_back(adaptationSet);
     }
 }
 
-BaseAdaptationSet *BasePeriod::getAdaptationSetByID(const ID &id)
+BaseAdaptationSet *BasePeriod::getAdaptationSetByID(const adaptive::ID &id)
 {
     std::vector<BaseAdaptationSet*>::const_iterator it;
     for(it = adaptationSets.begin(); it!= adaptationSets.end(); ++it)
