@@ -31,7 +31,7 @@
 #include "BaseAdaptationSet.h"
 #include "SegmentTemplate.h"
 #include "SegmentTimeline.h"
-#include "ID.hpp"
+#include "../ID.hpp"
 
 using namespace adaptive;
 using namespace adaptive::playlist;
@@ -53,6 +53,11 @@ StreamFormat BaseRepresentation::getStreamFormat() const
     return StreamFormat();
 }
 
+BaseAdaptationSet * BaseRepresentation::getAdaptationSet()
+{
+    return adaptationSet;
+}
+
 uint64_t     BaseRepresentation::getBandwidth            () const
 {
     return bandwidth;
@@ -110,9 +115,9 @@ mtime_t BaseRepresentation::getMinAheadTime(uint64_t curnum) const
         const MediaSegmentTemplate *templ = dynamic_cast<MediaSegmentTemplate *>(seglist.front());
         if(templ)
         {
-            const uint64_t timescale = templ->inheritTimescale();
+            const Timescale timescale = templ->inheritTimescale();
             stime_t i_length = templ->getMinAheadScaledTime(curnum);
-            return i_length * CLOCK_FREQ / timescale;
+            return timescale.ToTime(i_length);
         }
 
         /* should not happen */
@@ -120,12 +125,13 @@ mtime_t BaseRepresentation::getMinAheadTime(uint64_t curnum) const
     }
 
     mtime_t minTime = 0;
+    const Timescale timescale = inheritTimescale();
     std::vector<ISegment *>::const_iterator it;
     for(it = seglist.begin(); it != seglist.end(); ++it)
     {
         const ISegment *seg = *it;
         if(seg->getSequenceNumber() > curnum)
-            minTime += seg->duration.Get() * CLOCK_FREQ / inheritTimescale();
+            minTime += timescale.ToTime(seg->duration.Get());
     }
 
     return minTime;
@@ -150,6 +156,12 @@ std::string BaseRepresentation::contextualize(size_t, const std::string &compone
     return component;
 }
 
+bool BaseRepresentation::bwCompare(const BaseRepresentation *a,
+                                   const BaseRepresentation *b)
+{
+    return a->getBandwidth() < b->getBandwidth();
+}
+
 bool BaseRepresentation::validateCodec(const std::string &) const
 {
     return true;
