@@ -27,6 +27,7 @@
 #endif
 
 #include "Segment.h"
+#include "BaseAdaptationSet.h"
 #include "BaseRepresentation.h"
 #include "AbstractPlaylist.hpp"
 #include "SegmentChunk.hpp"
@@ -61,37 +62,33 @@ ISegment::~ISegment()
     assert(chunksuse.Get() == 0);
 }
 
-SegmentChunk * ISegment::getChunk(const std::string &url, HTTPConnectionManager *connManager)
-{
-    HTTPChunkBufferedSource *source = new HTTPChunkBufferedSource(url, connManager);
-    if(startByte != endByte)
-        source->setBytesRange(BytesRange(startByte, endByte));
-    connManager->downloader->schedule(source);
-    return new (std::nothrow) SegmentChunk(this, source);
-}
-
 void ISegment::onChunkDownload(block_t **, SegmentChunk *, BaseRepresentation *)
 {
 
 }
 
-SegmentChunk* ISegment::toChunk(size_t index, BaseRepresentation *ctxrep, HTTPConnectionManager *connManager)
+SegmentChunk* ISegment::toChunk(size_t index, BaseRepresentation *rep, AbstractConnectionManager *connManager)
 {
-    SegmentChunk *chunk;
-    try
+    const std::string url = getUrlSegment().toString(index, rep);
+    HTTPChunkBufferedSource *source = new (std::nothrow) HTTPChunkBufferedSource(url, connManager,
+                                                                                 rep->getAdaptationSet()->getID());
+    if( source )
     {
-        chunk = getChunk(getUrlSegment().toString(index, ctxrep), connManager);
-        if (!chunk)
-            return NULL;
+        if(startByte != endByte)
+            source->setBytesRange(BytesRange(startByte, endByte));
+
+        SegmentChunk *chunk = new (std::nothrow) SegmentChunk(this, source, rep);
+        if( chunk )
+        {
+            connManager->start(source);
+            return chunk;
+        }
+        else
+        {
+            delete source;
+        }
     }
-    catch (int)
-    {
-        return NULL;
-    };
-
-    chunk->setRepresentation(ctxrep);
-
-    return chunk;
+    return NULL;
 }
 
 bool ISegment::isTemplate() const
@@ -130,6 +127,7 @@ void ISegment::debug(vlc_object_t *obj, int indent) const
         ss << " @" << startByte << ".." << endByte;
     if(startTime.Get() > 0)
     	 ss << " stime " << startTime.Get();
+    ss << " duration " << duration.Get();
     msg_Dbg(obj, "%s", ss.str().c_str());
 }
 
@@ -145,18 +143,21 @@ int ISegment::compare(ISegment *other) const
 {
     if(duration.Get())
     {
-        stime_t diff = startTime.Get() - other->startTime.Get();
-        if(diff)
-            return diff / diff;
+        if(startTime.Get() > other->startTime.Get())
+            return 1;
+        else if(startTime.Get() < other->startTime.Get())
+            return -1;
     }
 
-    size_t diff = startByte - other->startByte;
-    if(diff)
-        return diff / diff;
+    if(startByte > other->startByte)
+        return 1;
+    else if(startByte < other->startByte)
+        return -1;
 
-    diff = endByte - other->endByte;
-    if(diff)
-        return diff / diff;
+    if(endByte > other->endByte)
+        return 1;
+    else if(endByte < other->endByte)
+        return -1;
 
     return 0;
 }
@@ -261,7 +262,7 @@ IndexSegment::IndexSegment(ICanonicalUrl *parent) :
 }
 
 SubSegment::SubSegment(ISegment *main, size_t start, size_t end) :
-    ISegment(main), parent(main)
+    ISegment(main)
 {
     setByteRange(start, end);
     debugName = "SubSegment";
