@@ -44,11 +44,11 @@ namespace adaptive
                 const std::vector<ISegment *>&   getSegments() const;
                 ISegment *              getSegmentByNumber(uint64_t);
                 void                    addSegment(ISegment *seg);
-                void                    mergeWith(SegmentList *);
+                void                    mergeWith(SegmentList *, bool = false);
                 void                    pruneBySegmentNumber(uint64_t);
                 void                    pruneByPlaybackTime(mtime_t);
                 bool                    getSegmentNumberByScaledTime(stime_t, uint64_t *) const;
-                mtime_t                 getPlaybackTimeBySegmentNumber(uint64_t);
+                bool                    getPlaybackTimeDurationBySegmentNumber(uint64_t, mtime_t *, mtime_t *) const;
 
             private:
                 std::vector<ISegment *>  segments;
