@@ -29,26 +29,29 @@ namespace hls
     class HLSStream : public AbstractStream
     {
         public:
-            HLSStream(demux_t *, const StreamFormat &);
-            virtual bool setPosition(mtime_t, bool); /* reimpl */
+            HLSStream(demux_t *);
+            virtual ~HLSStream();
 
         protected:
-            virtual AbstractDemuxer * createDemux(const StreamFormat &); /* reimpl */
-            virtual bool restartDemux(); /* reimpl */
-            virtual void prepareFormatChange(); /* reimpl */
-
+            virtual void setTimeOffset(mtime_t); /* reimpl */
             virtual block_t *checkBlock(block_t *, bool); /* reimpl */
+            virtual AbstractDemuxer * newDemux(demux_t *, const StreamFormat &,
+                                               es_out_t *, AbstractSourceStream *) const; /* reimpl */
 
         private:
-            bool b_timestamps_offset_set;
-            mtime_t i_aac_offset;
+            static int ID3TAG_Parse_Handler(uint32_t, const uint8_t *, size_t, void *);
+            int ParseID3Tag(uint32_t, const uint8_t *, size_t);
+            int ParseID3PrivTag(const uint8_t *, size_t);
+            bool b_id3_timestamps_offset_set;
+            vlc_meta_t *p_meta;
+            bool b_meta_updated;
     };
 
     class HLSStreamFactory : public AbstractStreamFactory
     {
         public:
             virtual AbstractStream *create(demux_t*, const StreamFormat &,
-                                   SegmentTracker *, HTTPConnectionManager *) const;
+                                   SegmentTracker *, AbstractConnectionManager *) const;
     };
 
 }
