@@ -2,7 +2,7 @@
  * itml.c : iTunes Music Library import functions
  *******************************************************************************
  * Copyright (C) 2007 VLC authors and VideoLAN
- * $Id: 36341d36e6397b8547948cbc134090b1c6ffaec8 $
+ * $Id: ebeb0eec113d990ca15dade30c62800dcda6827c $
  *
  * Authors: Yoann Peronneau <yoann@videolan.org>
  *
@@ -101,7 +101,10 @@ static int ReadDir( stream_t *p_demux, input_item_node_t *p_subitems )
     }
 
     xml_elem_hnd_t pl_elements[] =
-        { {"dict",    COMPLEX_CONTENT, {.cmplx = parse_plist_dict} } };
+        {
+            {"dict",    COMPLEX_CONTENT, {.cmplx = parse_plist_dict} },
+            {NULL,      UNKNOWN_CONTENT, {NULL} }
+        };
     parse_plist_node( p_demux, p_subitems, NULL, p_xml_reader, "plist",
                       pl_elements );
 
