@@ -26,33 +26,17 @@
 
 using namespace smooth;
 
-SmoothStream::SmoothStream(demux_t *demux, const StreamFormat &format)
-    :AbstractStream(demux, format)
+SmoothStream::SmoothStream(demux_t *demux)
+    :AbstractStream(demux)
 {
 }
 
-AbstractDemuxer * SmoothStream::createDemux(const StreamFormat &format)
+AbstractDemuxer *SmoothStream::newDemux(demux_t *p_realdemux, const StreamFormat &format,
+                                        es_out_t *out, AbstractSourceStream *source) const
 {
-    AbstractDemuxer *ret = NULL;
-    switch((unsigned)format)
-    {
-        case StreamFormat::MP4:
-            ret = new Demuxer(p_realdemux, "mp4", fakeesout->getEsOut(), demuxersource);
-            break;
-
-        default:
-        case StreamFormat::UNSUPPORTED:
-            break;
-    }
-
-    if(ret && !ret->create())
-    {
-        delete ret;
-        ret = NULL;
-    }
-    else fakeesout->commandsqueue.Commit();
-
-    return ret;
+    if((unsigned)format != StreamFormat::MP4)
+        return NULL;
+    return AbstractStream::newDemux(p_realdemux, format, out, source);
 }
 
 block_t * SmoothStream::checkBlock(block_t *p_block, bool)
@@ -61,15 +45,13 @@ block_t * SmoothStream::checkBlock(block_t *p_block, bool)
 }
 
 AbstractStream * SmoothStreamFactory::create(demux_t *realdemux, const StreamFormat &format,
-                                             SegmentTracker *tracker, HTTPConnectionManager *manager) const
+                                             SegmentTracker *tracker, AbstractConnectionManager *manager) const
 {
-    SmoothStream *stream;
-    try
+    SmoothStream *stream = new (std::nothrow) SmoothStream(realdemux);
+    if(stream && !stream->init(format,tracker, manager))
     {
-        stream = new SmoothStream(realdemux, format);
-    } catch (int) {
+        delete stream;
         return NULL;
     }
-    stream->bind(tracker, manager);
     return stream;
 }
