@@ -2,7 +2,7 @@
  * voc.c : Creative Voice File (.VOC) demux module for vlc
  *****************************************************************************
  * Copyright (C) 2005 Rémi Denis-Courmont
- * $Id: 9b0f8940ecf6c9b952318a11d95ae70ab4178fe1 $
+ * $Id: f5bcd52e02afe2ed0dda0f00a2f48a49803e761d $
  *
  * Authors: Rémi Denis-Courmont <rem # videolan.org>
  *
@@ -80,7 +80,7 @@ static int Open( vlc_object_t * p_this )
     const uint8_t *p_buf;
     uint16_t    i_data_offset, i_version;
 
-    if( stream_Peek( p_demux->s, &p_buf, 26 ) < 26 )
+    if( vlc_stream_Peek( p_demux->s, &p_buf, 26 ) < 26 )
         return VLC_EGENERIC;
 
     if( memcmp( p_buf, ct_header, 20 ) )
@@ -105,7 +105,7 @@ static int Open( vlc_object_t * p_this )
              i_version & 0xff );
 
     /* skip VOC header */
-    if( stream_Read( p_demux->s, NULL, i_data_offset ) < i_data_offset )
+    if( vlc_stream_Read( p_demux->s, NULL, i_data_offset ) < i_data_offset )
         return VLC_EGENERIC;
 
     p_demux->pf_demux   = Demux;
@@ -165,7 +165,7 @@ static int ReadBlockHeader( demux_t *p_demux )
     int32_t i_block_size;
     demux_sys_t *p_sys = p_demux->p_sys;
 
-    if( stream_Read( p_demux->s, buf, 4 ) < 4 )
+    if( vlc_stream_Read( p_demux->s, buf, 4 ) < 4 )
         return VLC_EGENERIC; /* EOF */
 
     i_block_size = GetDWLE( buf ) >> 8;
@@ -176,7 +176,7 @@ static int ReadBlockHeader( demux_t *p_demux )
 
     switch( *buf )
     {
-        case 0: /* not possible : caught with earlier stream_Read */
+        case 0: /* not possible : caught with earlier vlc_stream_Read */
             goto corrupt;
 
         case 1:
@@ -184,7 +184,7 @@ static int ReadBlockHeader( demux_t *p_demux )
                 goto corrupt;
             i_block_size -= 2;
 
-            if( stream_Read( p_demux->s, buf, 2 ) < 2 )
+            if( vlc_stream_Read( p_demux->s, buf, 2 ) < 2 )
                 goto corrupt;
 
             switch( buf[1] ) /* codec id */
@@ -251,7 +251,7 @@ static int ReadBlockHeader( demux_t *p_demux )
 
         case 3: /* silence block */
             if( ( i_block_size != 3 )
-             || ( stream_Read( p_demux->s, buf, 3 ) < 3 ) )
+             || ( vlc_stream_Read( p_demux->s, buf, 3 ) < 3 ) )
                 goto corrupt;
 
             i_block_size = 0;
@@ -269,12 +269,12 @@ static int ReadBlockHeader( demux_t *p_demux )
 
         case 6: /* repeat block */
             if( ( i_block_size != 2 )
-             || ( stream_Read( p_demux->s, buf, 2 ) < 2 ) )
+             || ( vlc_stream_Read( p_demux->s, buf, 2 ) < 2 ) )
                 goto corrupt;
 
             i_block_size = 0;
             p_sys->i_loop_count = GetWLE( buf );
-            p_sys->i_loop_offset = stream_Tell( p_demux->s );
+            p_sys->i_loop_offset = vlc_stream_Tell( p_demux->s );
             break;
 
         case 7: /* repeat end block */
@@ -283,7 +283,7 @@ static int ReadBlockHeader( demux_t *p_demux )
 
             if( p_sys->i_loop_count > 0 )
             {
-                if( stream_Seek( p_demux->s, p_sys->i_loop_offset ) )
+                if( vlc_stream_Seek( p_demux->s, p_sys->i_loop_offset ) )
                     msg_Warn( p_demux, "cannot loop: seek failed" );
                 else
                     p_sys->i_loop_count--;
@@ -299,7 +299,7 @@ static int ReadBlockHeader( demux_t *p_demux )
              * is used for stereo rather than 8
              */
             if( ( i_block_size != 4 )
-             || ( stream_Read( p_demux->s, buf, 4 ) < 4 ) )
+             || ( vlc_stream_Read( p_demux->s, buf, 4 ) < 4 ) )
                 goto corrupt;
 
             if( buf[2] )
@@ -309,6 +309,8 @@ static int ReadBlockHeader( demux_t *p_demux )
             }
 
             new_fmt.i_codec = VLC_CODEC_U8;
+            if (buf[3] >= 32)
+                goto corrupt;
             new_fmt.audio.i_channels = buf[3] + 1; /* can't be nul */
             new_fmt.audio.i_rate = 256000000L /
                           ((65536L - GetWLE(buf)) * new_fmt.audio.i_channels);
@@ -319,7 +321,7 @@ static int ReadBlockHeader( demux_t *p_demux )
             new_fmt.i_bitrate = new_fmt.audio.i_rate * 8;
 
             /* read subsequent block 1 */
-            if( stream_Read( p_demux->s, buf, 4 ) < 4 )
+            if( vlc_stream_Read( p_demux->s, buf, 4 ) < 4 )
                 return VLC_EGENERIC; /* EOF */
 
             i_block_size = GetDWLE( buf ) >> 8;
@@ -329,7 +331,7 @@ static int ReadBlockHeader( demux_t *p_demux )
                 goto corrupt;
             i_block_size -= 2;
 
-            if( stream_Read( p_demux->s, buf, 2 ) < 2 )
+            if( vlc_stream_Read( p_demux->s, buf, 2 ) < 2 )
                 goto corrupt;
 
             if( buf[1] )
@@ -345,8 +347,8 @@ static int ReadBlockHeader( demux_t *p_demux )
                 goto corrupt;
             i_block_size -= 12;
 
-            if( ( stream_Read( p_demux->s, buf, 8 ) < 8 )
-             || ( stream_Read( p_demux->s, NULL, 4 ) < 4 ) )
+            if( ( vlc_stream_Read( p_demux->s, buf, 8 ) < 8 )
+             || ( vlc_stream_Read( p_demux->s, NULL, 4 ) < 4 ) )
                 goto corrupt;
 
             new_fmt.audio.i_rate = GetDWLE( buf );
@@ -413,15 +415,17 @@ static int ReadBlockHeader( demux_t *p_demux )
         default:
             msg_Dbg( p_demux, "unknown block type %u - skipping block",
                      (unsigned)*buf);
+            /* fall through */
         case 4: /* blocks of non-audio types can be skipped */
         case 5:
-            if( stream_Read( p_demux->s, NULL, i_block_size ) < i_block_size )
+            if( vlc_stream_Read( p_demux->s, NULL,
+                                 i_block_size ) < i_block_size )
                 goto corrupt;
             i_block_size = 0;
             break;
     }
 
-    p_sys->i_block_start = stream_Tell( p_demux->s );
+    p_sys->i_block_start = vlc_stream_Tell( p_demux->s );
     p_sys->i_block_end = p_sys->i_block_start + i_block_size;
 
     if( i_block_size || p_sys->i_silence_countdown )
@@ -470,7 +474,7 @@ static int Demux( demux_t *p_demux )
 
     if( p_sys->i_silence_countdown == 0 )
     {
-        int64_t i_offset = stream_Tell( p_demux->s );
+        int64_t i_offset = vlc_stream_Tell( p_demux->s );
         if( i_offset >= p_sys->i_block_end )
         {
             if( ReadBlockHeader( p_demux ) != VLC_SUCCESS )
@@ -484,8 +488,9 @@ static int Demux( demux_t *p_demux )
         if( i_read_frames > MAX_READ_FRAME )
             i_read_frames = MAX_READ_FRAME;
 
-        p_block = stream_Block( p_demux->s,
-                                p_sys->fmt.audio.i_bytes_per_frame * i_read_frames );
+        p_block = vlc_stream_Block( p_demux->s,
+                                    p_sys->fmt.audio.i_bytes_per_frame
+                                    * i_read_frames );
         if( p_block == NULL )
         {
             msg_Warn( p_demux, "cannot read data" );
@@ -509,7 +514,7 @@ static int Demux( demux_t *p_demux )
     p_block->i_dts = p_block->i_pts = VLC_TS_0 + date_Get( &p_sys->pts );
     p_block->i_nb_samples = i_read_frames * p_sys->fmt.audio.i_frame_length;
     date_Increment( &p_sys->pts, p_block->i_nb_samples );
-    es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_block->i_pts );
+    es_out_SetPCR( p_demux->out, p_block->i_pts );
     es_out_Send( p_demux->out, p_sys->p_es, p_block );
 
     return 1;
