@@ -1,8 +1,8 @@
 /*****************************************************************************
  * applescript.h: MacOS X AppleScript support
  *****************************************************************************
- * Copyright (C) 2002-2003, 2005, 2007 VLC authors and VideoLAN
- * $Id: fc396400a228248d013bc6770e6e119a5c996212 $
+ * Copyright (C) 2002-2012 VLC authors and VideoLAN
+ * $Id: aee356bfaeedd9b03f1ab1a0b2544577cc5fc7f3 $
  *
  * Authors: Derk-Jan Hartman <thedj@users.sourceforge.net>
  *
@@ -40,17 +40,13 @@
 *****************************************************************************/
 @interface NSApplication(ScriptSupport)
 
-- (BOOL)scriptFullscreenMode;
-- (void)setScriptFullscreenMode: (BOOL)mode;
-
-- (int)audioVolume;
-- (void)setAudioVolume: (int)mode;
-
-- (int)currentTime;
-- (void)setCurrentTime: (int)mode;
-
-- (int) durationOfCurrentItem;
-- (NSString*) pathOfCurrentItem;
-- (NSString*) nameOfCurrentItem;
+@property (readwrite) BOOL scriptFullscreenMode;
+@property (readwrite) int audioVolume;
+@property (readwrite) int audioDesync;
+@property (readwrite) int currentTime;
+@property (readonly) int durationOfCurrentItem;
+@property (readonly) NSString *pathOfCurrentItem;
+@property (readonly) NSString *nameOfCurrentItem;
+@property (readonly) BOOL playbackShowsMenu;
 
 @end
