@@ -2,7 +2,7 @@
  * ogg.c: ogg muxer module for vlc
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2006 VLC authors and VideoLAN
- * $Id: 72d843c7f7fab41b6e31376253296da8a45cb7fa $
+ * $Id: 3847cc54d681062a577cd28cdff0b83e38bcdcb8 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -840,7 +840,10 @@ static void OggGetSkeletonFisbone( uint8_t **pp_buffer, long *pi_size,
 
     /* preroll */
     if ( p_input->p_fmt->p_extra )
-        SetDWLE( &(*pp_buffer)[44], xiph_CountHeaders( p_input->p_fmt->p_extra, p_input->p_fmt->i_extra ) );
+        SetDWLE( &(*pp_buffer)[44],
+                xiph_CountUnknownHeaders( p_input->p_fmt->p_extra,
+                                          p_input->p_fmt->i_extra,
+                                          p_input->p_fmt->i_codec ) );
 
     if ( headers.i_size > 0 )
     {
@@ -941,7 +944,6 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
     ogg_packet op;
     ogg_stream_t *p_stream;
     sout_mux_sys_t *p_sys = p_mux->p_sys;
-    int i;
 
     if( sout_AccessOutControl( p_mux->p_access,
                                ACCESS_OUT_CAN_SEEK,
@@ -997,7 +999,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
      * must appear first in the ogg stream so we take care of them first. */
     for( int pass = 0; pass < 2; pass++ )
     {
-        for( i = 0; i < p_mux->i_nb_inputs; i++ )
+        for( int i = 0; i < p_mux->i_nb_inputs; i++ )
         {
             sout_input_t *p_input = p_mux->pp_inputs[i];
             p_stream = (ogg_stream_t*)p_input->p_sys;
@@ -1024,8 +1026,9 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
             {
                 /* First packet in order: vorbis/speex/opus/theora/daala info */
                 unsigned pi_size[XIPH_MAX_HEADER_COUNT];
-                void     *pp_data[XIPH_MAX_HEADER_COUNT];
+                const void *pp_data[XIPH_MAX_HEADER_COUNT];
                 unsigned i_count;
+
                 if( xiph_SplitHeaders( pi_size, pp_data, &i_count,
                                        p_input->p_fmt->i_extra, p_input->p_fmt->p_extra ) )
                 {
@@ -1035,7 +1038,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 }
 
                 op.bytes  = pi_size[0];
-                op.packet = pp_data[0];
+                op.packet = (void *)pp_data[0];
                 if( pi_size[0] <= 0 )
                     msg_Err( p_mux, "header data corrupted");
 
@@ -1116,7 +1119,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
     /* Create fisbones if any */
     if ( p_sys->skeleton.b_create )
     {
-        for( i = 0; i < p_mux->i_nb_inputs; i++ )
+        for( int i = 0; i < p_mux->i_nb_inputs; i++ )
         {
             sout_input_t *p_input = p_mux->pp_inputs[i];
             ogg_stream_t *p_stream = (ogg_stream_t*)p_input->p_sys;
@@ -1145,7 +1148,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
     p_hdr = NULL;
 
     /* Create indexes if any */
-    for( i = 0; i < p_mux->i_nb_inputs; i++ )
+    for( int i = 0; i < p_mux->i_nb_inputs; i++ )
     {
         sout_input_t *p_input = p_mux->pp_inputs[i];
         ogg_stream_t *p_stream = (ogg_stream_t*)p_input->p_sys;
@@ -1178,7 +1181,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
     }
 
     /* Take care of the non b_o_s headers */
-    for( i = 0; i < p_mux->i_nb_inputs; i++ )
+    for( int i = 0; i < p_mux->i_nb_inputs; i++ )
     {
         sout_input_t *p_input = p_mux->pp_inputs[i];
         ogg_stream_t *p_stream = (ogg_stream_t*)p_input->p_sys;
@@ -1190,19 +1193,20 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
             p_stream->fmt.i_codec == VLC_CODEC_DAALA )
         {
             unsigned pi_size[XIPH_MAX_HEADER_COUNT];
-            void     *pp_data[XIPH_MAX_HEADER_COUNT];
+            const void *pp_data[XIPH_MAX_HEADER_COUNT];
             unsigned i_count;
+
             if( xiph_SplitHeaders( pi_size, pp_data, &i_count,
                                    p_input->p_fmt->i_extra, p_input->p_fmt->p_extra ) )
                 i_count = 0;
 
             /* Special case, headers are already there in the incoming stream.
              * We need to gather them an mark them as headers. */
-            for( unsigned i = 1; i < i_count; i++ )
+            for( unsigned j = 1; j < i_count; j++ )
             {
-                op.bytes  = pi_size[i];
-                op.packet = pp_data[i];
-                if( pi_size[i] <= 0 )
+                op.bytes  = pi_size[j];
+                op.packet = (void *)pp_data[j];
+                if( pi_size[j] <= 0 )
                     msg_Err( p_mux, "header data corrupted");
 
                 op.b_o_s  = 0;
@@ -1211,7 +1215,7 @@ static bool OggCreateHeaders( sout_mux_t *p_mux )
                 op.packetno = p_stream->i_packet_no++;
                 ogg_stream_packetin( &p_stream->os, &op );
                 msg_Dbg( p_mux, "adding non bos, secondary header" );
-                if( i == i_count - 1 )
+                if( j == i_count - 1 )
                     p_og = OggStreamFlush( p_mux, &p_stream->os, 0 );
                 else
                     p_og = OggStreamPageOut( p_mux, &p_stream->os, 0 );
