@@ -2,7 +2,7 @@
  * mpeg4audio.c: parse and packetize an MPEG 4 audio stream
  *****************************************************************************
  * Copyright (C) 2001, 2002, 2006 VLC authors and VideoLAN
- * $Id: 6391cd79dc1c7c7c962777f6441b9d7ef804b037 $
+ * $Id: b3a12ab2e91b5302effad54a0799aa77de605866 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -69,12 +69,13 @@ typedef struct
     {
         int i_object_type;
         int i_samplerate;
+        int i_channel;
     } extension;
 
     /* GASpecific */
     int i_frame_length;   // 1024 or 960
 
-} mpeg4_cfg_t;
+} mpeg4_asc_t;
 
 #define LATM_MAX_EXTRA_SIZE 64
 typedef struct
@@ -86,7 +87,7 @@ typedef struct
     int i_frame_length;         // type 1
     int i_frame_length_index;   // type 3 4 5 6 7
 
-    mpeg4_cfg_t cfg;
+    mpeg4_asc_t cfg;
 
     /* Raw configuration */
     int     i_extra;
@@ -109,7 +110,7 @@ typedef struct
     int i_streams;
     latm_stream_t stream[LATM_MAX_PROGRAM*LATM_MAX_LAYER];
 
-    int i_other_data;
+    uint32_t i_other_data;
     int i_crc;  /* -1 if not set */
 } latm_mux_t;
 
@@ -128,6 +129,7 @@ struct decoder_sys_t
      */
     date_t  end_date;
     mtime_t i_pts;
+    bool b_discontuinity;
 
     int i_frame_size;
     unsigned int i_channels;
@@ -156,8 +158,9 @@ enum
     } while(0)
 
 enum {
-    TYPE_NONE,
-    TYPE_RAW,
+    TYPE_UNKNOWN, /* AAC samples with[out] headers */
+    TYPE_UNKNOWN_NONRAW, /* [un]packetized ADTS or LOAS */
+    TYPE_RAW,    /* RAW AAC frames */
     TYPE_ADTS,
     TYPE_LOAS
 };
@@ -177,10 +180,10 @@ static const int pi_sample_rates[16] =
 static int  OpenPacketizer(vlc_object_t *);
 static void ClosePacketizer(vlc_object_t *);
 
-static block_t *PacketizeRawBlock    (decoder_t *, block_t **);
-static void     FlushRawBlock( decoder_t * );
-static block_t *PacketizeStreamBlock(decoder_t *, block_t **);
-static void     FlushStreamBlock( decoder_t * );
+static block_t *Packetize    (decoder_t *, block_t **);
+static void     Flush( decoder_t * );
+
+static int Mpeg4ReadAudioSpecificConfig(bs_t *s, mpeg4_asc_t *p_cfg, bool);
 
 /*****************************************************************************
  * Module descriptor
@@ -210,70 +213,90 @@ static int OpenPacketizer(vlc_object_t *p_this)
 
     /* Misc init */
     p_sys->i_state = STATE_NOSYNC;
+    p_sys->b_discontuinity = false;
     date_Set(&p_sys->end_date, 0);
     block_BytestreamInit(&p_sys->bytestream);
     p_sys->b_latm_cfg = false;
     p_sys->i_warnings = 0;
 
     /* Set output properties */
-    p_dec->fmt_out.i_cat = AUDIO_ES;
     p_dec->fmt_out.i_codec = VLC_CODEC_MP4A;
 
     msg_Dbg(p_dec, "running MPEG4 audio packetizer");
 
-    if (p_dec->fmt_in.i_extra > 0) {
-        uint8_t *p_config = (uint8_t*)p_dec->fmt_in.p_extra;
-        int     i_index;
+    /*
+     * We need to handle 3 cases.
+     * Case 1 : RAW AAC samples without sync header
+     *          The demuxer shouldn't need packetizer, see next case.
+     * Case 2 : AAC samples with ADTS or LOAS/LATM header
+     *          Some mux (avi) can't distinguish the both
+     *          cases above, and then forwards to packetizer
+     *          which should check for header and rewire to case below
+     * Case 3 : Non packetized ADTS or LOAS/LATM
+     *          The demuxer needs to set original_codec for hardwiring
+     */
 
-        i_index = ((p_config[0] << 1) | (p_config[1] >> 7)) & 0x0f;
-        if (i_index != 0x0f) {
-            p_dec->fmt_out.audio.i_rate = pi_sample_rates[i_index];
-            p_dec->fmt_out.audio.i_frame_length =
-                ((p_config[1] >> 2) & 0x01) ? 960 : 1024;
-        } else {
-            p_dec->fmt_out.audio.i_rate = ((p_config[1] & 0x7f) << 17) |
-                (p_config[2] << 9) | (p_config[3] << 1) |
-                (p_config[4] >> 7);
-            p_dec->fmt_out.audio.i_frame_length =
-                ((p_config[4] >> 2) & 0x01) ? 960 : 1024;
+    switch (p_dec->fmt_in.i_original_fourcc)
+    {
+        case VLC_FOURCC('L','A','T','M'):
+            p_sys->i_type = TYPE_LOAS;
+            msg_Dbg(p_dec, "LOAS/LATM Mode");
+            break;
+
+        case VLC_FOURCC('A','D','T','S'):
+            p_sys->i_type = TYPE_ADTS;
+            msg_Dbg(p_dec, "ADTS Mode");
+            break;
+
+        case VLC_FOURCC('H','E','A','D'):
+            p_sys->i_type = TYPE_UNKNOWN_NONRAW;
+            break;
+
+        default:
+            p_sys->i_type = TYPE_UNKNOWN;
+            break;
+    }
+
+    /* Some mux (avi) do send RAW AAC without extradata,
+       and LATM can be sent with out-of-band audioconfig,
+       (avformat sets m4a extradata in both cases)
+       so we can't rely on extradata to guess multiplexing */
+    p_dec->fmt_out.audio.i_rate = p_dec->fmt_in.audio.i_rate;
+
+    if(p_dec->fmt_in.i_extra)
+    {
+        mpeg4_asc_t asc;
+        bs_t s;
+        bs_init(&s, p_dec->fmt_in.p_extra, p_dec->fmt_in.i_extra);
+        if(Mpeg4ReadAudioSpecificConfig(&s, &asc, true) == VLC_SUCCESS)
+        {
+            p_dec->fmt_out.audio.i_rate = asc.i_samplerate;
+            p_dec->fmt_out.audio.i_frame_length = asc.i_frame_length;
+            p_dec->fmt_out.audio.i_channels = asc.i_channel;
+
+            msg_Dbg(p_dec, "%sAAC%s %dHz %d samples/frame",
+                    (asc.i_sbr) ? "HE-" : "",
+                    (asc.i_ps) ? "v2" : "",
+                    (asc.i_sbr) ? p_dec->fmt_out.audio.i_rate << 1
+                                : p_dec->fmt_out.audio.i_rate,
+                    p_dec->fmt_out.audio.i_frame_length);
         }
 
-        p_dec->fmt_out.audio.i_channels =
-            (p_config[i_index == 0x0f ? 4 : 1] >> 3) & 0x0f;
-
-        msg_Dbg(p_dec, "AAC %dHz %d samples/frame",
-                 p_dec->fmt_out.audio.i_rate,
-                 p_dec->fmt_out.audio.i_frame_length);
-
-        date_Init(&p_sys->end_date, p_dec->fmt_out.audio.i_rate, 1);
-
-        p_dec->fmt_out.i_extra = p_dec->fmt_in.i_extra;
         p_dec->fmt_out.p_extra = malloc(p_dec->fmt_in.i_extra);
-        if (!p_dec->fmt_out.p_extra) {
-            p_dec->fmt_out.i_extra = 0;
+        if (!p_dec->fmt_out.p_extra)
             return VLC_ENOMEM;
-        }
+        p_dec->fmt_out.i_extra = p_dec->fmt_in.i_extra;
         memcpy(p_dec->fmt_out.p_extra, p_dec->fmt_in.p_extra,
                 p_dec->fmt_in.i_extra);
-
-        /* Set callback */
-        p_dec->pf_packetize = PacketizeRawBlock;
-        p_dec->pf_flush = FlushRawBlock;
-        p_sys->i_type = TYPE_RAW;
-    } else {
-        msg_Dbg(p_dec, "no decoder specific info, must be an ADTS or LOAS stream");
-
-        date_Init(&p_sys->end_date, p_dec->fmt_in.audio.i_rate, 1);
-
-        /* We will try to create a AAC Config from adts/loas */
-        p_dec->fmt_out.i_extra = 0;
-        p_dec->fmt_out.p_extra = NULL;
-
-        /* Set callback */
-        p_dec->pf_packetize = PacketizeStreamBlock;
-        p_dec->pf_flush = FlushStreamBlock;
-        p_sys->i_type = TYPE_NONE;
     }
+    /* else() We will try to create a AAC Config from adts/loas */
+
+    date_Init(&p_sys->end_date, p_dec->fmt_out.audio.i_rate ?
+                                p_dec->fmt_out.audio.i_rate : 48000, 1);
+
+    /* Set callbacks */
+    p_dec->pf_packetize = Packetize;
+    p_dec->pf_flush = Flush;
 
     return VLC_SUCCESS;
 }
@@ -290,22 +313,12 @@ static void ClosePacketizer(vlc_object_t *p_this)
     free(p_sys);
 }
 
-/*****************************************************************************
- * FlushRawBlock:
- *****************************************************************************/
-static void FlushRawBlock(decoder_t *p_dec)
-{
-    decoder_sys_t *p_sys = p_dec->p_sys;
-
-    date_Set(&p_sys->end_date, 0);
-}
-
 /****************************************************************************
- * PacketizeRawBlock: the whole thing
+ * ForwardRawBlock:
  ****************************************************************************
  * This function must be fed with complete frames.
  ****************************************************************************/
-static block_t *PacketizeRawBlock(decoder_t *p_dec, block_t **pp_block)
+static block_t *ForwardRawBlock(decoder_t *p_dec, block_t **pp_block)
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
     block_t *p_block;
@@ -313,30 +326,30 @@ static block_t *PacketizeRawBlock(decoder_t *p_dec, block_t **pp_block)
     if (!pp_block || !*pp_block)
         return NULL;
 
-    if ((*pp_block)->i_flags & (BLOCK_FLAG_DISCONTINUITY | BLOCK_FLAG_CORRUPTED)) {
-        FlushRawBlock(p_dec);
-        if ((*pp_block)->i_flags&(BLOCK_FLAG_CORRUPTED)) {
-            block_Release(*pp_block);
-            return NULL;
-        }
-    }
-
     p_block = *pp_block;
     *pp_block = NULL; /* Don't reuse this block */
 
-    if (!date_Get(&p_sys->end_date) && p_block->i_pts <= VLC_TS_INVALID) {
-        /* We've just started the stream, wait for the first PTS. */
-        block_Release(p_block);
-        return NULL;
-    } else if (p_block->i_pts > VLC_TS_INVALID &&
-             p_block->i_pts != date_Get(&p_sys->end_date)) {
+    int64_t i_diff = 0;
+    if (p_block->i_pts > VLC_TS_INVALID &&
+        p_block->i_pts != date_Get(&p_sys->end_date))
+    {
+        if(date_Get(&p_sys->end_date) > VLC_TS_INVALID)
+            i_diff = llabs( date_Get(&p_sys->end_date) - p_block->i_pts );
         date_Set(&p_sys->end_date, p_block->i_pts);
     }
 
     p_block->i_pts = p_block->i_dts = date_Get(&p_sys->end_date);
 
-    p_block->i_length = date_Increment(&p_sys->end_date,
-        p_dec->fmt_out.audio.i_frame_length) - p_block->i_pts;
+    /* Might not be known due to missing extradata,
+       will be set to block pts above */
+    if(p_dec->fmt_out.audio.i_frame_length && p_block->i_pts != VLC_TS_INVALID)
+    {
+        p_block->i_length = date_Increment(&p_sys->end_date,
+            p_dec->fmt_out.audio.i_frame_length) - p_block->i_pts;
+
+        if( i_diff > p_block->i_length )
+            p_sys->b_discontuinity = true;
+    }
 
     return p_block;
 }
@@ -477,9 +490,13 @@ static int Mpeg4GAProgramConfigElement(bs_t *s)
     return 0;
 }
 
-static int Mpeg4GASpecificConfig(mpeg4_cfg_t *p_cfg, bs_t *s)
+static int Mpeg4GASpecificConfig(mpeg4_asc_t *p_cfg, bs_t *s)
 {
     p_cfg->i_frame_length = bs_read1(s) ? 960 : 1024;
+    if(p_cfg->i_object_type == 23) /* 14496-3 4.5.1.1 */
+        p_cfg->i_frame_length >>= 1;
+    else if(p_cfg->i_object_type == 3)
+        p_cfg->i_frame_length = 256;
 
     if (bs_read1(s))     // depend on core coder
         bs_skip(s, 14);   // core coder delay
@@ -518,7 +535,17 @@ static int Mpeg4ReadAudioSamplerate(bs_t *s)
     return bs_read(s, 24);
 }
 
-static int Mpeg4ReadAudioSpecificInfo(mpeg4_cfg_t *p_cfg, int *pi_extra, uint8_t *p_extra, bs_t *s, int i_max_size)
+static int Mpeg4ReadAudioChannelConfiguration(bs_t *s)
+{
+    int i_channel = bs_read(s, 4);
+    if (i_channel == 7)
+        i_channel = 8; // 7.1
+    else if (i_channel >= 8)
+        i_channel = -1;
+    return i_channel;
+}
+
+static int Mpeg4ReadAudioSpecificConfig(bs_t *s, mpeg4_asc_t *p_cfg, bool b_withext)
 {
 #if 0
     static const char *ppsz_otype[] = {
@@ -540,27 +567,17 @@ static int Mpeg4ReadAudioSpecificInfo(mpeg4_cfg_t *p_cfg, int *pi_extra, uint8_t
         "DST",
     };
 #endif
-    const int i_pos_start = bs_pos(s);
-    bs_t s_sav = *s;
-    int i_bits;
-    int i;
-
     memset(p_cfg, 0, sizeof(*p_cfg));
-    *pi_extra = 0;
 
     p_cfg->i_object_type = Mpeg4ReadAudioObjectType(s);
     p_cfg->i_samplerate = Mpeg4ReadAudioSamplerate(s);
-
-    p_cfg->i_channel = bs_read(s, 4);
-    if (p_cfg->i_channel == 7)
-        p_cfg->i_channel = 8; // 7.1
-    else if (p_cfg->i_channel >= 8)
-        p_cfg->i_channel = -1;
+    p_cfg->i_channel = Mpeg4ReadAudioChannelConfiguration(s);
 
     p_cfg->i_sbr = -1;
     p_cfg->i_ps  = -1;
     p_cfg->extension.i_object_type = 0;
     p_cfg->extension.i_samplerate = 0;
+    p_cfg->extension.i_channel = -1;
     if (p_cfg->i_object_type == 5 || p_cfg->i_object_type == 29) {
         p_cfg->i_sbr = 1;
         if (p_cfg->i_object_type == 29)
@@ -569,6 +586,8 @@ static int Mpeg4ReadAudioSpecificInfo(mpeg4_cfg_t *p_cfg, int *pi_extra, uint8_t
         p_cfg->extension.i_samplerate = Mpeg4ReadAudioSamplerate(s);
 
         p_cfg->i_object_type = Mpeg4ReadAudioObjectType(s);
+        if(p_cfg->i_object_type == 22)
+            p_cfg->extension.i_channel = Mpeg4ReadAudioChannelConfiguration(s);
     }
 
     switch(p_cfg->i_object_type)
@@ -580,45 +599,41 @@ static int Mpeg4ReadAudioSpecificInfo(mpeg4_cfg_t *p_cfg, int *pi_extra, uint8_t
         break;
     case 8:
         // CelpSpecificConfig();
-        break;
     case 9:
         // HvxcSpecificConfig();
-        break;
     case 12:
         // TTSSSpecificConfig();
-        break;
     case 13: case 14: case 15: case 16:
         // StructuredAudioSpecificConfig();
-        break;
     case 24:
         // ERCelpSpecificConfig();
-        break;
     case 25:
         // ERHvxcSpecificConfig();
-        break;
     case 26: case 27:
         // ParametricSpecificConfig();
-        break;
     case 28:
         // SSCSpecificConfig();
-        break;
     case 32: case 33: case 34:
         // MPEG_1_2_SpecificConfig();
-        break;
     case 35:
         // DSTSpecificConfig();
-        break;
     case 36:
         // ALSSpecificConfig();
-        break;
+    case 37: case 38:
+        // SLSSpecificConfig();
+    case 39:
+        // ELDSpecificConfig();
+    case 40: case 41:
+        // SymbolicMusicSpecificConfig();
     default:
         // error
-        break;
+        return VLC_EGENERIC;
     }
+
     switch(p_cfg->i_object_type)
     {
     case 17: case 19: case 20: case 21: case 22: case 23:
-    case 24: case 25: case 26: case 27:
+    case 24: case 25: case 26: case 27: case 39:
     {
         int epConfig = bs_read(s, 2);
         if (epConfig == 2 || epConfig == 3)
@@ -633,42 +648,51 @@ static int Mpeg4ReadAudioSpecificInfo(mpeg4_cfg_t *p_cfg, int *pi_extra, uint8_t
         break;
     }
 
-    if (p_cfg->extension.i_object_type != 5 && i_max_size > 0 && i_max_size - (bs_pos(s) - i_pos_start) >= 16 &&
+    if (b_withext && p_cfg->extension.i_object_type != 5 && bs_remain(s) >= 16 &&
         bs_read(s, 11) == 0x2b7) {
         p_cfg->extension.i_object_type = Mpeg4ReadAudioObjectType(s);
-        if (p_cfg->extension.i_object_type == 5) {
+        if (p_cfg->extension.i_object_type == 5)
+        {
             p_cfg->i_sbr  = bs_read1(s);
             if (p_cfg->i_sbr == 1) {
                 p_cfg->extension.i_samplerate = Mpeg4ReadAudioSamplerate(s);
-                if (i_max_size > 0 && i_max_size - (bs_pos(s) - i_pos_start) >= 12 && bs_read(s, 11) == 0x548)
+                if (bs_remain(s) >= 12 && bs_read(s, 11) == 0x548)
                    p_cfg->i_ps = bs_read1(s);
             }
         }
+        else if (p_cfg->extension.i_object_type == 22)
+        {
+            p_cfg->i_sbr  = bs_read1(s);
+            if(p_cfg->i_sbr)
+                p_cfg->extension.i_samplerate = Mpeg4ReadAudioSamplerate(s);
+            p_cfg->extension.i_channel = Mpeg4ReadAudioChannelConfiguration(s);
+        }
     }
 
     //fprintf(stderr, "Mpeg4ReadAudioSpecificInfo: t=%s(%d)f=%d c=%d sbr=%d\n",
     //         ppsz_otype[p_cfg->i_object_type], p_cfg->i_object_type, p_cfg->i_samplerate, p_cfg->i_channel, p_cfg->i_sbr);
 
-    i_bits = bs_pos(s) - i_pos_start;
-
-    *pi_extra = __MIN((i_bits + 7) / 8, LATM_MAX_EXTRA_SIZE);
-    for (i = 0; i < *pi_extra; i++) {
-        const int i_read = __MIN(8, i_bits - 8*i);
-        p_extra[i] = bs_read(&s_sav, i_read) << (8-i_read);
-    }
-    return i_bits;
+    return VLC_SUCCESS;
 }
 
-static int LatmGetValue(bs_t *s)
+static uint32_t LatmGetValue(bs_t *s)
 {
-    int i_bytes = bs_read(s, 2);
-    int v = 0;
-    for (int i = 0; i < i_bytes; i++)
+    uint32_t v = 0;
+    for (int i = 1 + bs_read(s, 2); i > 0; i--)
         v = (v << 8) + bs_read(s, 8);
-
     return v;
 }
 
+static uint32_t AudioSpecificConfigBitsToBytes(bs_t *s, uint32_t i_bits, uint8_t *p_data)
+{
+    uint32_t i_extra = __MIN((i_bits + 7) / 8, LATM_MAX_EXTRA_SIZE);
+    for (uint32_t i = 0; i < i_extra; i++) {
+        const uint32_t i_read = __MIN(8, i_bits - 8*i);
+        p_data[i] = bs_read(s, i_read) << (8-i_read);
+    }
+    return i_extra;
+}
+
 static int LatmReadStreamMuxConfiguration(latm_mux_t *m, bs_t *s)
 {
     int i_mux_version;
@@ -711,12 +735,16 @@ static int LatmReadStreamMuxConfiguration(latm_mux_t *m, bs_t *s)
                 assert(m->i_streams > 0);
                 st->cfg = m->stream[m->i_streams-1].cfg;
             } else {
-                int i_cfg_size = 0;
-                if (i_mux_version == 1)
-                    i_cfg_size = LatmGetValue(s);
-                i_cfg_size -= Mpeg4ReadAudioSpecificInfo(&st->cfg, &st->i_extra, st->extra, s, i_cfg_size);
-                if (i_cfg_size > 0)
-                    bs_skip(s, i_cfg_size);
+                uint32_t asc_size = 0;
+                if(i_mux_version > 0)
+                    asc_size = LatmGetValue(s);
+                bs_t asc_bs = *s;
+                Mpeg4ReadAudioSpecificConfig(&asc_bs, &st->cfg, i_mux_version > 0);
+                if (i_mux_version == 0)
+                    asc_size = bs_pos(&asc_bs) - bs_pos(s);
+                asc_bs = *s;
+                st->i_extra = AudioSpecificConfigBitsToBytes(&asc_bs, asc_size, st->extra);
+                bs_skip(s, asc_size);
             }
 
             st->i_frame_length_type = bs_read(s, 3);
@@ -781,28 +809,48 @@ static int LOASParse(decoder_t *p_dec, uint8_t *p_buffer, int i_buffer)
             p_sys->latm.i_streams > 0) {
         const latm_stream_t *st = &p_sys->latm.stream[0];
 
+        if(st->cfg.i_samplerate <= 0 || st->cfg.i_channel <=0 || st->cfg.i_frame_length <= 0)
+            return 0;
+
         p_sys->i_channels = st->cfg.i_channel;
         p_sys->i_rate = st->cfg.i_samplerate;
         p_sys->i_frame_length = st->cfg.i_frame_length;
 
-        /* FIXME And if it changes ? */
-        if (p_sys->i_channels && p_sys->i_rate && p_sys->i_frame_length > 0) {
-            if (!p_dec->fmt_out.i_extra && st->i_extra > 0) {
-                p_dec->fmt_out.i_extra = st->i_extra;
+        if (p_sys->i_channels && p_sys->i_rate && p_sys->i_frame_length > 0)
+        {
+            if(p_dec->fmt_out.i_extra != st->i_extra ||
+               (p_dec->fmt_out.i_extra > 0 &&
+                memcmp(p_dec->fmt_out.p_extra, st->extra, st->i_extra)) )
+            {
+                if(p_dec->fmt_out.i_extra)
+                    free(p_dec->fmt_out.p_extra);
                 p_dec->fmt_out.p_extra = malloc(st->i_extra);
-                if (!p_dec->fmt_out.p_extra) {
+                if(p_dec->fmt_out.p_extra)
+                {
+                    p_dec->fmt_out.i_extra = st->i_extra;
+                    memcpy(p_dec->fmt_out.p_extra, st->extra, st->i_extra);
+                    p_sys->b_latm_cfg = true;
+                }
+                else
+                {
                     p_dec->fmt_out.i_extra = 0;
-                    return 0;
+                    p_sys->b_latm_cfg = false;
                 }
-                memcpy(p_dec->fmt_out.p_extra, st->extra, st->i_extra);
             }
-            p_sys->b_latm_cfg = true;
         }
     }
 
     /* Wait for the configuration */
     if (!p_sys->b_latm_cfg)
-        return 0;
+    {
+        /* WAVE_FORMAT_MPEG_LOAS, configuration provided as AAC header :/ */
+        if( p_dec->fmt_in.i_extra > 0 &&
+            p_sys->i_channels && p_sys->i_rate && p_sys->i_frame_length )
+        {
+            p_sys->b_latm_cfg = true;
+        }
+        else return 0;
+    }
 
     /* FIXME do we need to split the subframe into independent packet ? */
     if (p_sys->latm.i_sub_frames > 1)
@@ -920,7 +968,8 @@ static void SetupOutput(decoder_t *p_dec, block_t *p_block)
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
 
-    if (p_dec->fmt_out.audio.i_rate != p_sys->i_rate) {
+    if (p_dec->fmt_out.audio.i_rate != p_sys->i_rate && p_sys->i_rate > 0)
+    {
         msg_Info(p_dec, "AAC channels: %d samplerate: %d",
                   p_sys->i_channels, p_sys->i_rate);
 
@@ -935,7 +984,6 @@ static void SetupOutput(decoder_t *p_dec, block_t *p_block)
     p_dec->fmt_out.audio.i_frame_length = p_sys->i_frame_length;
 
 #if 0
-    p_dec->fmt_out.audio.i_original_channels = p_sys->i_channels_conf;
     p_dec->fmt_out.audio.i_physical_channels = p_sys->i_channels_conf;
 #endif
 
@@ -948,13 +996,24 @@ static void SetupOutput(decoder_t *p_dec, block_t *p_block)
 /*****************************************************************************
  * FlushStreamBlock:
  *****************************************************************************/
-static void FlushStreamBlock(decoder_t *p_dec)
+static void Flush(decoder_t *p_dec)
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
 
     p_sys->i_state = STATE_NOSYNC;
     block_BytestreamEmpty(&p_sys->bytestream);
     date_Set(&p_sys->end_date, VLC_TS_INVALID);
+    p_sys->b_discontuinity = true;
+}
+
+static inline bool HasADTSHeader( const uint8_t *p_header )
+{
+    return p_header[0] == 0xff && (p_header[1] & 0xf6) == 0xf0;
+}
+
+static inline bool HasLoasHeader( const uint8_t *p_header )
+{
+    return p_header[0] == 0x56 && (p_header[1] & 0xe0) == 0xe0;
 }
 
 /****************************************************************************
@@ -967,37 +1026,31 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
     block_t *p_out_buffer;
     uint8_t *p_buf;
 
-    if (!pp_block || !*pp_block)
-        return NULL;
+    block_t *p_block = pp_block ? *pp_block : NULL;
 
-    if ((*pp_block)->i_flags & (BLOCK_FLAG_DISCONTINUITY|BLOCK_FLAG_CORRUPTED)) {
-        FlushStreamBlock(p_dec);
-        if ((*pp_block)->i_flags & BLOCK_FLAG_CORRUPTED) {
-            block_Release(*pp_block);
-            return NULL;
-        }
+    if(p_block)
+    {
+        block_BytestreamPush(&p_sys->bytestream, p_block);
+        *pp_block = NULL;
     }
 
-    if (!date_Get(&p_sys->end_date) && (*pp_block)->i_pts <= VLC_TS_INVALID) {
-        /* We've just started the stream, wait for the first PTS. */
-        block_Release(*pp_block);
-        return NULL;
-    }
-
-    block_BytestreamPush(&p_sys->bytestream, *pp_block);
-
     for (;;) switch(p_sys->i_state) {
     case STATE_NOSYNC:
         while (block_PeekBytes(&p_sys->bytestream, p_header, 2) == VLC_SUCCESS) {
             /* Look for sync word - should be 0xfff(adts) or 0x2b7(loas) */
-            if (p_header[0] == 0xff && (p_header[1] & 0xf6) == 0xf0) {
+            if ((p_sys->i_type == TYPE_ADTS || p_sys->i_type == TYPE_UNKNOWN_NONRAW) &&
+                HasADTSHeader( p_header ) )
+            {
                 if (p_sys->i_type != TYPE_ADTS)
                     msg_Dbg(p_dec, "detected ADTS format");
 
                 p_sys->i_state = STATE_SYNC;
                 p_sys->i_type = TYPE_ADTS;
                 break;
-            } else if (p_header[0] == 0x56 && (p_header[1] & 0xe0) == 0xe0) {
+            }
+            else if ((p_sys->i_type == TYPE_LOAS || p_sys->i_type == TYPE_UNKNOWN_NONRAW) &&
+                      HasLoasHeader( p_header ) )
+            {
                 if (p_sys->i_type != TYPE_LOAS)
                     msg_Dbg(p_dec, "detected LOAS format");
 
@@ -1013,12 +1066,13 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
             /* Need more data */
             return NULL;
         }
+        /* fallthrough */
 
     case STATE_SYNC:
         /* New frame, set the Presentation Time Stamp */
         p_sys->i_pts = p_sys->bytestream.p_block->i_pts;
         if (p_sys->i_pts > VLC_TS_INVALID &&
-                p_sys->i_pts != date_Get(&p_sys->end_date))
+            p_sys->i_pts != date_Get(&p_sys->end_date))
             date_Set(&p_sys->end_date, p_sys->i_pts);
         p_sys->i_state = STATE_HEADER;
         break;
@@ -1055,10 +1109,9 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
         }
 
         p_sys->i_state = STATE_NEXT_SYNC;
+        /* fallthrough */
 
     case STATE_NEXT_SYNC:
-        /* TODO: If p_block == NULL, flush the buffer without checking the
-         * next sync word */
         if (p_sys->bytestream.p_block == NULL) {
             p_sys->i_state = STATE_NOSYNC;
             block_BytestreamFlush(&p_sys->bytestream);
@@ -1068,17 +1121,37 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
         /* Check if next expected frame contains the sync word */
         if (block_PeekOffsetBytes(&p_sys->bytestream, p_sys->i_frame_size
                     + p_sys->i_header_size, p_header, 2) != VLC_SUCCESS)
+        {
+            if(p_block == NULL) /* drain */
+            {
+                p_sys->i_state = STATE_SEND_DATA;
+                break;
+            }
             return NULL; /* Need more data */
+        }
 
         assert((p_sys->i_type == TYPE_ADTS) || (p_sys->i_type == TYPE_LOAS));
-        if (((p_sys->i_type == TYPE_ADTS) &&
-                    (p_header[0] != 0xff || (p_header[1] & 0xf6) != 0xf0)) ||
-                ((p_sys->i_type == TYPE_LOAS) &&
-                 (p_header[0] != 0x56 || (p_header[1] & 0xe0) != 0xe0))) {
-            msg_Dbg(p_dec, "emulated sync word "
-                    "(no sync on following frame)");
-            p_sys->i_state = STATE_NOSYNC;
-            block_SkipByte(&p_sys->bytestream);
+        if ( (p_sys->i_type == TYPE_ADTS && !HasADTSHeader( p_header )) ||
+             (p_sys->i_type == TYPE_LOAS && !HasLoasHeader( p_header )) )
+        {
+            /* Check spacial padding case. Failing if need more bytes is ok since
+               that should have been sent as a whole block */
+            if( block_PeekOffsetBytes(&p_sys->bytestream,
+                                      p_sys->i_frame_size + p_sys->i_header_size,
+                                      p_header, 3) == VLC_SUCCESS &&
+                p_header[0] == 0x00 &&
+               ((p_sys->i_type == TYPE_ADTS && HasADTSHeader( &p_header[1] )) ||
+                (p_sys->i_type == TYPE_LOAS && !HasLoasHeader( &p_header[1] ))))
+            {
+                p_sys->i_state = STATE_SEND_DATA;
+            }
+            else
+            {
+                msg_Dbg(p_dec, "emulated sync word (no sync on following frame)"
+                               " 0x%"PRIx8" 0x%"PRIx8, p_header[0], p_header[1] );
+                p_sys->i_state = STATE_NOSYNC;
+                block_SkipByte(&p_sys->bytestream);
+            }
             break;
         }
 
@@ -1092,6 +1165,7 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
                     p_sys->i_header_size) != VLC_SUCCESS)
             return NULL; /* Need more data */
         p_sys->i_state = STATE_SEND_DATA;
+        /* fallthrough */
 
     case STATE_SEND_DATA:
         /* When we reach this point we already know we have enough
@@ -1099,7 +1173,6 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
 
         p_out_buffer = block_Alloc(p_sys->i_frame_size);
         if (!p_out_buffer) {
-            //p_dec->b_error = true;
             return NULL;
         }
         p_buf = p_out_buffer->p_buffer;
@@ -1112,7 +1185,8 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
         if (p_sys->i_type != TYPE_ADTS) { /* parse/extract the whole frame */
             assert(p_sys->i_type == TYPE_LOAS);
             p_out_buffer->i_buffer = LOASParse(p_dec, p_buf, p_sys->i_frame_size);
-            if (p_out_buffer->i_buffer <= 0) {
+            if (p_out_buffer->i_buffer <= 0)
+            {
                 if (!p_sys->b_latm_cfg)
                     msg_Warn(p_dec, "waiting for header");
 
@@ -1128,7 +1202,8 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
             p_sys->i_pts = p_sys->bytestream.p_block->i_pts = VLC_TS_INVALID;
 
         /* So p_block doesn't get re-added several times */
-        *pp_block = block_BytestreamPop(&p_sys->bytestream);
+        if( pp_block )
+            *pp_block = block_BytestreamPop(&p_sys->bytestream);
 
         p_sys->i_state = STATE_NOSYNC;
 
@@ -1137,3 +1212,71 @@ static block_t *PacketizeStreamBlock(decoder_t *p_dec, block_t **pp_block)
 
     return NULL;
 }
+
+/****************************************************************************
+ * Packetize: just forwards raw blocks, or packetizes LOAS/ADTS
+ *            and strips headers
+ ****************************************************************************/
+static block_t *Packetize(decoder_t *p_dec, block_t **pp_block)
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+    block_t *p_block = pp_block ? *pp_block : NULL;
+
+    if(p_block)
+    {
+        if (p_block->i_flags & (BLOCK_FLAG_DISCONTINUITY|BLOCK_FLAG_CORRUPTED))
+        {
+            if(p_sys->i_type == TYPE_ADTS || p_sys->i_type == TYPE_LOAS)
+            {
+                /* First always drain complete blocks before discontinuity */
+                block_t *p_drain = PacketizeStreamBlock(p_dec, NULL);
+                if(p_drain)
+                    return p_drain;
+            }
+
+            Flush(p_dec);
+
+            if (p_block->i_flags & BLOCK_FLAG_CORRUPTED)
+            {
+                block_Release(p_block);
+                return NULL;
+            }
+        }
+
+        if (!date_Get(&p_sys->end_date) && p_block->i_pts <= VLC_TS_INVALID)
+        {
+            /* We've just started the stream, wait for the first PTS. */
+            block_Release(p_block);
+            return NULL;
+        }
+    }
+
+    if(p_block && p_sys->i_type == TYPE_UNKNOWN)
+    {
+        p_sys->i_type = TYPE_RAW;
+        if(p_block->i_buffer > 1)
+        {
+            if(p_block->p_buffer[0] == 0xff && (p_block->p_buffer[1] & 0xf6) == 0xf0)
+            {
+                p_sys->i_type = TYPE_ADTS;
+            }
+            else if(p_block->p_buffer[0] == 0x56 && (p_block->p_buffer[1] & 0xe0) == 0xe0)
+            {
+                p_sys->i_type = TYPE_LOAS;
+            }
+        }
+    }
+
+    if(p_sys->i_type == TYPE_RAW)
+        p_block = ForwardRawBlock(p_dec, pp_block);
+    else
+        p_block = PacketizeStreamBlock(p_dec, pp_block);
+
+    if(p_block && p_sys->b_discontuinity)
+    {
+        p_block->i_flags |= BLOCK_FLAG_DISCONTINUITY;
+        p_sys->b_discontuinity = false;
+    }
+
+    return p_block;
+}
