@@ -12,7 +12,7 @@
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
@@ -95,10 +95,10 @@ static const char *ppsz_sout_options_lang[] = {
     "id", "lang", NULL
 };
 
-static sout_stream_id_t *AddId   ( sout_stream_t *, es_format_t * );
-static sout_stream_id_t *AddLang ( sout_stream_t *, es_format_t * );
-static int               Del     ( sout_stream_t *, sout_stream_id_t * );
-static int               Send    ( sout_stream_t *, sout_stream_id_t *, block_t * );
+static sout_stream_id_sys_t *AddId  ( sout_stream_t *, const es_format_t * );
+static sout_stream_id_sys_t *AddLang( sout_stream_t *, const es_format_t * );
+static void              Del     ( sout_stream_t *, sout_stream_id_sys_t * );
+static int               Send    ( sout_stream_t *, sout_stream_id_sys_t *, block_t * );
 
 struct sout_stream_sys_t
 {
@@ -185,41 +185,49 @@ static void Close( vlc_object_t * p_this )
     free( p_sys );
 }
 
-static sout_stream_id_t * AddId( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t * AddId( sout_stream_t *p_stream, const es_format_t *p_fmt )
 {
     sout_stream_sys_t *p_sys = (sout_stream_sys_t *)p_stream->p_sys;
+    es_format_t fmt;
 
-    if ( p_fmt->i_id == p_sys->i_id )
+    if( p_fmt->i_id == p_sys->i_id )
     {
-        msg_Dbg( p_stream, "turning ID %d to %d",
-                 p_sys->i_id, p_sys->i_new_id );
-        p_fmt->i_id = p_sys->i_new_id;
+        msg_Dbg( p_stream, "turning ID %d to %d", p_sys->i_id,
+                 p_sys->i_new_id );
+
+        fmt = *p_fmt;
+        fmt.i_id = p_sys->i_new_id;
+        p_fmt = &fmt;
     }
 
-    return p_stream->p_next->pf_add( p_stream->p_next, p_fmt );
+    return sout_StreamIdAdd( p_stream->p_next, p_fmt );
 }
 
-static sout_stream_id_t * AddLang( sout_stream_t *p_stream, es_format_t *p_fmt )
+static sout_stream_id_sys_t * AddLang( sout_stream_t *p_stream, const es_format_t *p_fmt )
 {
     sout_stream_sys_t *p_sys = (sout_stream_sys_t *)p_stream->p_sys;
+    es_format_t fmt;
 
     if ( p_fmt->i_id == p_sys->i_id )
     {
         msg_Dbg( p_stream, "turning language %s of ID %d to %s",
                  p_fmt->psz_language ? p_fmt->psz_language : "unk",
                  p_sys->i_id, p_sys->psz_language );
-        p_fmt->psz_language = strdup( p_sys->psz_language );
+
+        fmt = *p_fmt;
+        fmt.psz_language = p_sys->psz_language;
+        p_fmt = &fmt;
     }
 
-    return p_stream->p_next->pf_add( p_stream->p_next, p_fmt );
+    return sout_StreamIdAdd( p_stream->p_next, p_fmt );
 }
 
-static int Del( sout_stream_t *p_stream, sout_stream_id_t *id )
+static void Del( sout_stream_t *p_stream, sout_stream_id_sys_t *id )
 {
-    return p_stream->p_next->pf_del( p_stream->p_next, id );
+    p_stream->p_next->pf_del( p_stream->p_next, id );
 }
 
-static int Send( sout_stream_t *p_stream, sout_stream_id_t *id,
+static int Send( sout_stream_t *p_stream, sout_stream_id_sys_t *id,
                  block_t *p_buffer )
 {
     return p_stream->p_next->pf_send( p_stream->p_next, id, p_buffer );
