@@ -2,7 +2,7 @@
  * algo_basic.c : Basic algorithms for the VLC deinterlacer
  *****************************************************************************
  * Copyright (C) 2000-2011 VLC authors and VideoLAN
- * $Id: 8511d891819f0033bb6ceaa4ca4c5542e360e1eb $
+ * $Id: 6ad9b06e0cb4d37bd7042f2b81b74abe08fa29b4 $
  *
  * Author: Sam Hocevar <sam@zoy.org>
  *         Damien Lucas <nitrox@videolan.org>  (Bob, Blend)
@@ -28,6 +28,7 @@
 #endif
 
 #include <stdint.h>
+#include <assert.h>
 
 #include <vlc_common.h>
 #include <vlc_picture.h>
@@ -42,8 +43,9 @@
  * RenderDiscard: only keep TOP or BOTTOM field, discard the other.
  *****************************************************************************/
 
-void RenderDiscard( picture_t *p_outpic, picture_t *p_pic, int i_field )
+int RenderDiscard( filter_t *p_filter, picture_t *p_outpic, picture_t *p_pic )
 {
+    VLC_UNUSED(p_filter);
     int i_plane;
 
     /* Copy image and skip lines */
@@ -51,8 +53,7 @@ void RenderDiscard( picture_t *p_outpic, picture_t *p_pic, int i_field )
     {
         uint8_t *p_in, *p_out_end, *p_out;
 
-        p_in = p_pic->p[i_plane].p_pixels
-                   + i_field * p_pic->p[i_plane].i_pitch;
+        p_in = p_pic->p[i_plane].p_pixels;
 
         p_out = p_outpic->p[i_plane].p_pixels;
         p_out_end = p_out + p_outpic->p[i_plane].i_pitch
@@ -66,14 +67,18 @@ void RenderDiscard( picture_t *p_outpic, picture_t *p_pic, int i_field )
             p_in += 2 * p_pic->p[i_plane].i_pitch;
         }
     }
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * RenderBob: renders a BOB picture - simple copy
  *****************************************************************************/
 
-void RenderBob( picture_t *p_outpic, picture_t *p_pic, int i_field )
+int RenderBob( filter_t *p_filter, picture_t *p_outpic, picture_t *p_pic,
+               int order, int i_field )
 {
+    VLC_UNUSED(p_filter);
+    VLC_UNUSED(order);
     int i_plane;
 
     /* Copy image and skip lines */
@@ -118,15 +123,18 @@ void RenderBob( picture_t *p_outpic, picture_t *p_pic, int i_field )
             memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
         }
     }
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * RenderLinear: BOB with linear interpolation
  *****************************************************************************/
 
-void RenderLinear( filter_t *p_filter,
-                   picture_t *p_outpic, picture_t *p_pic, int i_field )
+int RenderLinear( filter_t *p_filter,
+                  picture_t *p_outpic, picture_t *p_pic, int order, int i_field )
 {
+    VLC_UNUSED(p_filter);
+    VLC_UNUSED(order);
     int i_plane;
 
     /* Copy image and skip lines */
@@ -173,15 +181,16 @@ void RenderLinear( filter_t *p_filter,
         }
     }
     EndMerge();
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * RenderMean: Half-resolution blender
  *****************************************************************************/
 
-void RenderMean( filter_t *p_filter,
-                 picture_t *p_outpic, picture_t *p_pic )
+int RenderMean( filter_t *p_filter, picture_t *p_outpic, picture_t *p_pic )
 {
+    VLC_UNUSED(p_filter);
     int i_plane;
 
     /* Copy image and skip lines */
@@ -206,46 +215,43 @@ void RenderMean( filter_t *p_filter,
         }
     }
     EndMerge();
+    return VLC_SUCCESS;
 }
 
 /*****************************************************************************
  * RenderBlend: Full-resolution blender
  *****************************************************************************/
 
-void RenderBlend( filter_t *p_filter,
-                  picture_t *p_outpic, picture_t *p_pic )
+int RenderBlend( filter_t *p_filter, picture_t *p_outpic, picture_t *p_pic )
 {
+    VLC_UNUSED(p_filter);
     int i_plane;
-    vlc_chroma_description_t* p_desc = p_filter->p_sys->chroma;
 
     /* Copy image and skip lines */
     for( i_plane = 0 ; i_plane < p_pic->i_planes ; i_plane++ )
     {
         uint8_t *p_in, *p_out_end, *p_out;
-        size_t i_x_offset = p_pic->format.i_x_offset * p_desc->p[i_plane].w.num /
-                p_desc->p[i_plane].w.den * p_pic->p[i_plane].i_pixel_pitch;
-        size_t i_y_offset = p_pic->format.i_y_offset * p_desc->p[i_plane].h.num /
-                p_desc->p[i_plane].h.den * p_pic->p[i_plane].i_pitch;
 
-        p_in = p_pic->p[i_plane].p_pixels + i_y_offset;
+        p_in = p_pic->p[i_plane].p_pixels;
 
-        p_out = p_outpic->p[i_plane].p_pixels + i_y_offset;
+        p_out = p_outpic->p[i_plane].p_pixels;
         p_out_end = p_out + p_outpic->p[i_plane].i_pitch
                              * p_outpic->p[i_plane].i_visible_lines;
 
         /* First line: simple copy */
-        memcpy( p_out + i_x_offset, p_in + i_x_offset, p_pic->p[i_plane].i_visible_pitch );
+        memcpy( p_out, p_in, p_pic->p[i_plane].i_pitch );
         p_out += p_outpic->p[i_plane].i_pitch;
 
         /* Remaining lines: mean value */
         for( ; p_out < p_out_end ; )
         {
-            Merge( p_out + i_x_offset, p_in + i_x_offset, p_in + p_pic->p[i_plane].i_pitch + i_x_offset,
-                   p_pic->p[i_plane].i_visible_pitch );
+            Merge( p_out, p_in, p_in + p_pic->p[i_plane].i_pitch,
+                   p_pic->p[i_plane].i_pitch );
 
             p_out += p_outpic->p[i_plane].i_pitch;
             p_in  += p_pic->p[i_plane].i_pitch;
         }
     }
     EndMerge();
+    return VLC_SUCCESS;
 }
