@@ -2,7 +2,7 @@
  * mirror.c : Mirror video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2009 VLC authors and VideoLAN
- * $Id: 1cf5778669d3e095ac0809466db33f4b97754258 $
+ * $Id: f6137c508a4e421fdfcc12eb4479b5d5a6c7dc9f $
  *
  * Authors: Branko Kokanovic <branko.kokanovic@gmail.com>
  *
@@ -35,6 +35,7 @@
 #include <vlc_plugin.h>
 #include <vlc_atomic.h>
 #include <vlc_filter.h>
+#include <vlc_picture.h>
 #include "filter_picture.h"
 
 /*****************************************************************************
@@ -61,14 +62,14 @@ static const char *const ppsz_filter_options[] = {
  * Module descriptor
  *****************************************************************************/
 #define ORIENTATION_TEXT N_("Mirror orientation")
-#define ORIENTATION_LONGTEXT N_("Defines orientation of the mirror splitting. \
-    Can be vertical or horizontal" )
+#define ORIENTATION_LONGTEXT N_("Defines orientation of the mirror splitting. " \
+    "Can be vertical or horizontal." )
 static const int pi_orientation_values[] = { 0, 1 };
 static const char *const ppsz_orientation_descriptions[] = {
   N_("Vertical"), N_("Horizontal") };
 
 #define DIRECTION_TEXT N_("Direction")
-#define DIRECTION_LONGTEXT N_("Direction of the mirroring" )
+#define DIRECTION_LONGTEXT N_("Direction of the mirroring." )
 static const int pi_direction_values[] = { 0, 1 };
 static const char *const ppsz_direction_descriptions[] = {
   N_("Left to right/Top to bottom"), N_("Right to left/Bottom to top") };
@@ -81,7 +82,7 @@ vlc_module_begin ()
     set_help( N_("Splits video in two same parts, like in a mirror") )
     set_category( CAT_VIDEO )
     set_subcategory( SUBCAT_VIDEO_VFILTER )
-    set_capability( "video filter2", 0 )
+    set_capability( "video filter", 0 )
     add_integer( CFG_PREFIX "split", 0, ORIENTATION_TEXT,
                 ORIENTATION_LONGTEXT, false )
         change_integer_list( pi_orientation_values,
