@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
  * Copyright (C) 2002-2007 Rémi Denis-Courmont
- * $Id: 5f5d0efdb222abc863cc9cd6ffeb299994debcbf $
+ * $Id: 90dbdda9744a66e05475bfeb0c1ccabdea764b1d $
  *
  * Author: Rémi Denis-Courmont <rem # videolan.org>
  *
@@ -119,3 +119,14 @@ int vlc_getaddrinfo (const char *node, unsigned port,
 
     return getaddrinfo (node, servname, hints, res);
 }
+
+#if defined (_WIN32) || defined (__OS2__) \
+ || defined (__ANDROID__) || defined (__APPLE__) \
+ || defined (__native_client__)
+#warning vlc_getaddr_info_i11e() not implemented!
+int vlc_getaddrinfo_i11e(const char *node, unsigned port,
+                         const struct addrinfo *hints, struct addrinfo **res)
+{
+    return vlc_getaddrinfo(node, port, hints, res);
+}
+#endif
