@@ -4,7 +4,7 @@
  * Copyright (C) 2004-2006 VLC authors and VideoLAN
  * Copyright © 2006-2007 Rémi Denis-Courmont
  *
- * $Id: 72bd34620b4d34aec8090961fdb58d6ee463aee5 $
+ * $Id: f8d3bc9805953000f80066604fd1a04c18da044e $
  *
  * Authors: Laurent Aimar <fenrir@videolan.org>
  *          Rémi Denis-Courmont <rem # videolan.org>
@@ -104,8 +104,8 @@ static int net_SetupDgramSocket (vlc_object_t *p_obj, int fd,
      * SetSocketMediaStreamingMode is present in win 8 and later, so we set
      * receive buffer if that isn't present
      */
-#if _WIN32_WINNT < 0x602
-    HINSTANCE h_Network = LoadLibraryW(L"Windows.Networking.dll");
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN8)
+    HINSTANCE h_Network = LoadLibrary(TEXT("Windows.Networking.dll"));
     if( (h_Network == NULL) ||
         (GetProcAddress( h_Network, "SetSocketMediaStreamingMode" ) == NULL ) )
     {
@@ -316,7 +316,7 @@ net_SourceSubscribe (vlc_object_t *obj, int fd,
 /* MCAST_JOIN_SOURCE_GROUP was introduced to OS X in v10.7, but it doesn't work,
  * so ignore it to use the same code path as on 10.5 or 10.6 */
 #if defined (MCAST_JOIN_SOURCE_GROUP) && !defined (__APPLE__)
-    /* Agnostic SSM multicast join */
+    /* Family-agnostic Source-Specific Multicast join */
     int level;
     struct group_source_req gsr;
 
@@ -397,7 +397,7 @@ int net_Subscribe (vlc_object_t *obj, int fd,
 /* MCAST_JOIN_GROUP was introduced to OS X in v10.7, but it doesn't work,
  * so ignore it to use the same code as on 10.5 or 10.6 */
 #if defined (MCAST_JOIN_GROUP) && !defined (__APPLE__)
-    /* Agnostic SSM multicast join */
+    /* Family-agnostic Any-Source Multicast join */
     int level;
     struct group_req gr;
 
@@ -617,7 +617,7 @@ int net_OpenDgram( vlc_object_t *obj, const char *psz_bind, int i_bind,
     int val = vlc_getaddrinfo (psz_server, i_server, &hints, &rem);
     if (val)
     {
-        msg_Err (obj, "cannot resolve %s port %d : %s", psz_bind, i_bind,
+        msg_Err (obj, "cannot resolve %s port %d : %s", psz_server, i_server,
                  gai_strerror (val));
         return -1;
     }
