@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2001-2007 VLC authors and VideoLAN
  * Copyright (C) 2012 KO Myung-Hun
- * $Id: d24e26bd9b68285a229eabe41e336ae53395400a $
+ * $Id: c57180cfa7e247a74bf95d0929f9569b939f5ed9 $
  *
  * Authors: Sam Hocevar <sam@zoy.org>
  *          Ethan C. Baldridge <BaldridgeE@cadmus.com>
@@ -30,6 +30,8 @@
 # include "config.h"
 #endif
 
+#include <string.h>
+
 #include <vlc_common.h>
 #include <vlc_charset.h>
 #include "modules/modules.h"
@@ -90,5 +92,8 @@ void module_Unload( module_handle_t handle )
  */
 void *module_Lookup( module_handle_t handle, const char *psz_function )
 {
-    return dlsym( handle, psz_function );
+    char buf[strlen(psz_function) + 2];
+    buf[0] = '_';
+    strcpy(buf + 1, psz_function);
+    return dlsym( handle, buf );
 }
