@@ -33,7 +33,9 @@
 #include <windows.h>
 #include <wchar.h>
 
-#if (_WIN32_WINNT < 0x601)
+extern DWORD LoadLibraryFlags;
+
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN7)
 static BOOL WINAPI SetThreadErrorModeFallback(DWORD mode, DWORD *oldmode)
 {
     /* TODO: cache the pointer */
@@ -41,30 +43,26 @@ static BOOL WINAPI SetThreadErrorModeFallback(DWORD mode, DWORD *oldmode)
     if (unlikely(h == NULL))
         return FALSE;
 
-    BOOL WINAPI (*SetThreadErrorModeReal)(DWORD, DWORD *);
+    BOOL (WINAPI *SetThreadErrorModeReal)(DWORD, DWORD *);
 
     SetThreadErrorModeReal = GetProcAddress(h, "SetThreadErrorMode");
     if (SetThreadErrorModeReal != NULL)
         return SetThreadErrorModeReal(mode, oldmode);
 
-# if (_WIN32_WINNT >= 0x600)
-    DWORD curmode = GetErrorMode();
-# else
-    UINT WINAPI (*GetErrorModeReal)(void);
+# if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
+    /* As per libvlc_new() documentation, the calling process is responsible
+     * for setting a proper error mode on Windows 2008 and earlier versions.
+     * This is only a sanity check. */
+    UINT (WINAPI *GetErrorModeReal)(void);
     DWORD curmode = 0;
 
     GetErrorModeReal = (void *)GetProcAddress(h, "GetErrorMode");
     if (GetErrorModeReal != NULL)
         curmode = GetErrorModeReal();
     else
-    {
-        /* We are on XP, 2003, 2003/R2 or some special versions of Vista:
-           No SetThreadErrorMode, no GetErrorMode.
-           We will set the mode for the whole process, which is quite bad,
-           but is our only solution */
-        SetErrorMode( mode );
-        return TRUE;
-    }
+        curmode = SEM_FAILCRITICALERRORS;
+# else
+    DWORD curmode = GetErrorMode();
 # endif
     /* Extra flags should be OK. Missing flags are NOT OK. */
     if ((mode & curmode) != mode)
@@ -103,15 +101,14 @@ int module_Load( vlc_object_t *p_this, const char *psz_file,
     module_handle_t handle = NULL;
 #if !VLC_WINSTORE_APP
     DWORD mode;
-
     if (SetThreadErrorMode (SEM_FAILCRITICALERRORS, &mode) != 0)
-#endif
     {
-        handle = LoadLibraryW (wfile);
-#if !VLC_WINSTORE_APP
+        handle = LoadLibraryExW (wfile, NULL, LoadLibraryFlags );
         SetThreadErrorMode (mode, NULL);
-#endif
     }
+#else
+    handle = LoadPackagedLibrary( wfile, 0 );
+#endif
     free (wfile);
 
     if( handle == NULL )
