@@ -38,10 +38,13 @@
 #include <limits.h>
 #include <errno.h>
 #include <time.h>
+#if !VLC_WINSTORE_APP
+#include <mmsystem.h>
+#endif
 
 /*** Static mutex and condition variable ***/
-static vlc_mutex_t super_mutex;
-static vlc_cond_t  super_variable;
+static CRITICAL_SECTION super_mutex;
+static CONDITION_VARIABLE super_variable;
 
 #define IS_INTERRUPTIBLE (!VLC_WINSTORE_APP || _WIN32_WINNT >= 0x0A00)
 
@@ -66,6 +69,33 @@ struct vlc_thread
     } wait;
 };
 
+/*** Condition variables (low-level) ***/
+#if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
+static VOID (WINAPI *InitializeConditionVariable_)(PCONDITION_VARIABLE);
+#define InitializeConditionVariable InitializeConditionVariable_
+static BOOL (WINAPI *SleepConditionVariableCS_)(PCONDITION_VARIABLE,
+                                                PCRITICAL_SECTION, DWORD);
+#define SleepConditionVariableCS SleepConditionVariableCS_
+static VOID (WINAPI *WakeAllConditionVariable_)(PCONDITION_VARIABLE);
+#define WakeAllConditionVariable WakeAllConditionVariable_
+
+static void WINAPI DummyConditionVariable(CONDITION_VARIABLE *cv)
+{
+    (void) cv;
+}
+
+static BOOL WINAPI SleepConditionVariableFallback(CONDITION_VARIABLE *cv,
+                                                  CRITICAL_SECTION *cs,
+                                                  DWORD ms)
+{
+    (void) cv;
+    LeaveCriticalSection(cs);
+    SleepEx(ms > 5 ? 5 : ms, TRUE);
+    EnterCriticalSection(cs);
+    return ms != 0;
+}
+#endif
+
 /*** Mutexes ***/
 void vlc_mutex_init( vlc_mutex_t *p_mutex )
 {
@@ -92,19 +122,15 @@ void vlc_mutex_lock (vlc_mutex_t *p_mutex)
 {
     if (!p_mutex->dynamic)
     {   /* static mutexes */
-        int canc = vlc_savecancel ();
-        assert (p_mutex != &super_mutex); /* this one cannot be static */
-
-        vlc_mutex_lock (&super_mutex);
+        EnterCriticalSection(&super_mutex);
         while (p_mutex->locked)
         {
             p_mutex->contention++;
-            vlc_cond_wait (&super_variable, &super_mutex);
+            SleepConditionVariableCS(&super_variable, &super_mutex, INFINITE);
             p_mutex->contention--;
         }
         p_mutex->locked = true;
-        vlc_mutex_unlock (&super_mutex);
-        vlc_restorecancel (canc);
+        LeaveCriticalSection(&super_mutex);
         return;
     }
 
@@ -117,14 +143,13 @@ int vlc_mutex_trylock (vlc_mutex_t *p_mutex)
     {   /* static mutexes */
         int ret = EBUSY;
 
-        assert (p_mutex != &super_mutex); /* this one cannot be static */
-        vlc_mutex_lock (&super_mutex);
+        EnterCriticalSection(&super_mutex);
         if (!p_mutex->locked)
         {
             p_mutex->locked = true;
             ret = 0;
         }
-        vlc_mutex_unlock (&super_mutex);
+        LeaveCriticalSection(&super_mutex);
         return ret;
     }
 
@@ -135,14 +160,12 @@ void vlc_mutex_unlock (vlc_mutex_t *p_mutex)
 {
     if (!p_mutex->dynamic)
     {   /* static mutexes */
-        assert (p_mutex != &super_mutex); /* this one cannot be static */
-
-        vlc_mutex_lock (&super_mutex);
+        EnterCriticalSection(&super_mutex);
         assert (p_mutex->locked);
         p_mutex->locked = false;
         if (p_mutex->contention)
-            vlc_cond_broadcast (&super_variable);
-        vlc_mutex_unlock (&super_mutex);
+            WakeAllConditionVariable(&super_variable);
+        LeaveCriticalSection(&super_mutex);
         return;
     }
 
@@ -225,13 +248,13 @@ int vlc_threadvar_create (vlc_threadvar_t *p_tls, void (*destr) (void *))
     var->next = NULL;
     *p_tls = var;
 
-    vlc_mutex_lock (&super_mutex);
+    EnterCriticalSection(&super_mutex);
     var->prev = vlc_threadvar_last;
     if (var->prev)
         var->prev->next = var;
 
     vlc_threadvar_last = var;
-    vlc_mutex_unlock (&super_mutex);
+    LeaveCriticalSection(&super_mutex);
     return 0;
 }
 
@@ -239,7 +262,7 @@ void vlc_threadvar_delete (vlc_threadvar_t *p_tls)
 {
     struct vlc_threadvar *var = *p_tls;
 
-    vlc_mutex_lock (&super_mutex);
+    EnterCriticalSection(&super_mutex);
     if (var->prev != NULL)
         var->prev->next = var->next;
 
@@ -248,7 +271,7 @@ void vlc_threadvar_delete (vlc_threadvar_t *p_tls)
     else
         vlc_threadvar_last = var->prev;
 
-    vlc_mutex_unlock (&super_mutex);
+    LeaveCriticalSection(&super_mutex);
 
     TlsFree (var->id);
     free (var);
@@ -257,11 +280,12 @@ void vlc_threadvar_delete (vlc_threadvar_t *p_tls)
 int vlc_threadvar_set (vlc_threadvar_t key, void *value)
 {
     int saved = GetLastError ();
-    int val = TlsSetValue (key->id, value) ? ENOMEM : 0;
 
-    if (val == 0)
-        SetLastError(saved);
-    return val;
+    if (!TlsSetValue(key->id, value))
+        return ENOMEM;
+
+    SetLastError(saved);
+    return 0;
 }
 
 void *vlc_threadvar_get (vlc_threadvar_t key)
@@ -278,48 +302,21 @@ static void vlc_threadvars_cleanup(void)
     vlc_threadvar_t key;
 retry:
     /* TODO: use RW lock or something similar */
-    vlc_mutex_lock(&super_mutex);
+    EnterCriticalSection(&super_mutex);
     for (key = vlc_threadvar_last; key != NULL; key = key->prev)
     {
         void *value = vlc_threadvar_get(key);
         if (value != NULL && key->destroy != NULL)
         {
-            vlc_mutex_unlock(&super_mutex);
+            LeaveCriticalSection(&super_mutex);
             vlc_threadvar_set(key, NULL);
             key->destroy(value);
             goto retry;
         }
     }
-    vlc_mutex_unlock(&super_mutex);
+    LeaveCriticalSection(&super_mutex);
 }
 
-/*** Condition variables (low-level) ***/
-#if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
-static VOID (WINAPI *InitializeConditionVariable_)(PCONDITION_VARIABLE);
-#define InitializeConditionVariable InitializeConditionVariable_
-static BOOL (WINAPI *SleepConditionVariableCS_)(PCONDITION_VARIABLE,
-                                                PCRITICAL_SECTION, DWORD);
-#define SleepConditionVariableCS SleepConditionVariableCS_
-static VOID (WINAPI *WakeAllConditionVariable_)(PCONDITION_VARIABLE);
-#define WakeAllConditionVariable WakeAllConditionVariable_
-
-static void WINAPI DummyConditionVariable(CONDITION_VARIABLE *cv)
-{
-    (void) cv;
-}
-
-static BOOL WINAPI SleepConditionVariableFallback(CONDITION_VARIABLE *cv,
-                                                  CRITICAL_SECTION *cs,
-                                                  DWORD ms)
-{
-    (void) cv;
-    LeaveCriticalSection(cs);
-    SleepEx(ms > 5 ? 5 : ms, TRUE);
-    EnterCriticalSection(cs);
-    return ms != 0;
-}
-#endif
-
 /*** Futeces^WAddress waits ***/
 #if (_WIN32_WINNT < _WIN32_WINNT_WIN8)
 static BOOL (WINAPI *WaitOnAddress_)(VOID volatile *, PVOID, SIZE_T, DWORD);
@@ -677,6 +674,7 @@ void vlc_control_cancel (int cmd, ...)
             void *addr = va_arg(ap, void *);
 
             EnterCriticalSection(&th->wait.lock);
+            assert(th->wait.addr == NULL);
             th->wait.addr = addr;
             LeaveCriticalSection(&th->wait.lock);
             break;
@@ -699,13 +697,13 @@ void vlc_control_cancel (int cmd, ...)
 /*** Clock ***/
 static union
 {
-#if (_WIN32_WINNT < 0x0601)
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN7)
     struct
     {
         BOOL (*query) (PULONGLONG);
     } interrupt;
 #endif
-#if (_WIN32_WINNT < 0x0600)
+#if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
     struct
     {
         ULONGLONG (*get) (void);
@@ -715,6 +713,13 @@ static union
     {
         LARGE_INTEGER freq;
     } perf;
+#if !VLC_WINSTORE_APP
+    struct
+    {
+        MMRESULT (WINAPI *timeGetDevCaps)(LPTIMECAPS ptc,UINT cbtc);
+        DWORD (WINAPI *timeGetTime)(void);
+    } multimedia;
+#endif
 } clk;
 
 static mtime_t mdate_interrupt (void)
@@ -722,7 +727,7 @@ static mtime_t mdate_interrupt (void)
     ULONGLONG ts;
     BOOL ret;
 
-#if (_WIN32_WINNT >= 0x0601)
+#if (_WIN32_WINNT >= _WIN32_WINNT_WIN7)
     ret = QueryUnbiasedInterruptTime (&ts);
 #else
     ret = clk.interrupt.query (&ts);
@@ -737,7 +742,7 @@ static mtime_t mdate_interrupt (void)
 
 static mtime_t mdate_tick (void)
 {
-#if (_WIN32_WINNT >= 0x0600)
+#if (_WIN32_WINNT >= _WIN32_WINNT_VISTA)
     ULONGLONG ts = GetTickCount64 ();
 #else
     ULONGLONG ts = clk.tick.get ();
@@ -748,10 +753,9 @@ static mtime_t mdate_tick (void)
     return ts * (CLOCK_FREQ / 1000);
 }
 #if !VLC_WINSTORE_APP
-#include <mmsystem.h>
 static mtime_t mdate_multimedia (void)
 {
-     DWORD ts = timeGetTime ();
+    DWORD ts = clk.multimedia.timeGetTime ();
 
     /* milliseconds */
     static_assert ((CLOCK_FREQ % 1000) == 0, "Broken frequencies ratio");
@@ -778,7 +782,7 @@ static mtime_t mdate_wall (void)
     FILETIME ts;
     ULARGE_INTEGER s;
 
-#if (_WIN32_WINNT >= 0x0602) && !VLC_WINSTORE_APP
+#if (_WIN32_WINNT >= _WIN32_WINNT_WIN8) && (!VLC_WINSTORE_APP || _WIN32_WINNT >= 0x0A00)
     GetSystemTimePreciseAsFileTime (&ts);
 #else
     GetSystemTimeAsFileTime (&ts);
@@ -790,20 +794,9 @@ static mtime_t mdate_wall (void)
     return s.QuadPart / (10000000 / CLOCK_FREQ);
 }
 
-static CRITICAL_SECTION clock_lock;
-static bool clock_used_early = false;
-
 static mtime_t mdate_default(void)
 {
-    EnterCriticalSection(&clock_lock);
-    if (!clock_used_early)
-    {
-        if (!QueryPerformanceFrequency(&clk.perf.freq))
-            abort();
-        clock_used_early = true;
-    }
-    LeaveCriticalSection(&clock_lock);
-
+    vlc_threads_setup(NULL);
     return mdate_perf();
 }
 
@@ -837,32 +830,27 @@ void (msleep)(mtime_t delay)
 }
 #endif
 
-static void SelectClockSource (vlc_object_t *obj)
+static BOOL SelectClockSource(void *data)
 {
-    EnterCriticalSection (&clock_lock);
-    if (mdate_selected != mdate_default)
-    {
-        LeaveCriticalSection (&clock_lock);
-        return;
-    }
-
-    assert(!clock_used_early);
+    vlc_object_t *obj = data;
 
 #if VLC_WINSTORE_APP
     const char *name = "perf";
 #else
     const char *name = "multimedia";
 #endif
-    char *str = var_InheritString (obj, "clock-source");
+    char *str = NULL;
+    if (obj != NULL)
+        str = var_InheritString(obj, "clock-source");
     if (str != NULL)
         name = str;
     if (!strcmp (name, "interrupt"))
     {
         msg_Dbg (obj, "using interrupt time as clock source");
-#if (_WIN32_WINNT < 0x0601)
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN7)
         HANDLE h = GetModuleHandle (_T("kernel32.dll"));
         if (unlikely(h == NULL))
-            abort ();
+            return FALSE;
         clk.interrupt.query = (void *)GetProcAddress (h,
                                                       "QueryUnbiasedInterruptTime");
         if (unlikely(clk.interrupt.query == NULL))
@@ -874,13 +862,13 @@ static void SelectClockSource (vlc_object_t *obj)
     if (!strcmp (name, "tick"))
     {
         msg_Dbg (obj, "using Windows time as clock source");
-#if (_WIN32_WINNT < 0x0600)
+#if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
         HANDLE h = GetModuleHandle (_T("kernel32.dll"));
         if (unlikely(h == NULL))
-            abort ();
+            return FALSE;
         clk.tick.get = (void *)GetProcAddress (h, "GetTickCount64");
         if (unlikely(clk.tick.get == NULL))
-            abort ();
+            return FALSE;
 #endif
         mdate_selected = mdate_tick;
     }
@@ -889,18 +877,33 @@ static void SelectClockSource (vlc_object_t *obj)
     if (!strcmp (name, "multimedia"))
     {
         TIMECAPS caps;
+        MMRESULT (WINAPI * timeBeginPeriod)(UINT);
+
+        HMODULE hWinmm = LoadLibrary(TEXT("winmm.dll"));
+        if (!hWinmm)
+            goto perf;
+
+        clk.multimedia.timeGetDevCaps = (void*)GetProcAddress(hWinmm, "timeGetDevCaps");
+        clk.multimedia.timeGetTime = (void*)GetProcAddress(hWinmm, "timeGetTime");
+        if (!clk.multimedia.timeGetDevCaps || !clk.multimedia.timeGetTime)
+            goto perf;
 
         msg_Dbg (obj, "using multimedia timers as clock source");
-        if (timeGetDevCaps (&caps, sizeof (caps)) != MMSYSERR_NOERROR)
-            abort ();
+        if (clk.multimedia.timeGetDevCaps (&caps, sizeof (caps)) != MMSYSERR_NOERROR)
+            goto perf;
         msg_Dbg (obj, " min period: %u ms, max period: %u ms",
                  caps.wPeriodMin, caps.wPeriodMax);
         mdate_selected = mdate_multimedia;
+
+        timeBeginPeriod = (void*)GetProcAddress(hWinmm, "timeBeginPeriod");
+        if (timeBeginPeriod != NULL)
+            timeBeginPeriod(5);
     }
 #endif
     else
     if (!strcmp (name, "perf"))
     {
+    perf:
         msg_Dbg (obj, "using performance counters as clock source");
         if (!QueryPerformanceFrequency (&clk.perf.freq))
             abort ();
@@ -918,8 +921,8 @@ static void SelectClockSource (vlc_object_t *obj)
         msg_Err (obj, "invalid clock source \"%s\"", name);
         abort ();
     }
-    LeaveCriticalSection (&clock_lock);
     free (str);
+    return TRUE;
 }
 
 size_t EnumClockSource (vlc_object_t *obj, const char *var,
@@ -930,7 +933,7 @@ size_t EnumClockSource (vlc_object_t *obj, const char *var,
     char **names = xmalloc (sizeof (*names) * max);
     size_t n = 0;
 
-#if (_WIN32_WINNT < 0x0601)
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN7)
     DWORD version = LOWORD(GetVersion());
     version = (LOBYTE(version) << 8) | (HIBYTE(version) << 0);
 #endif
@@ -938,7 +941,7 @@ size_t EnumClockSource (vlc_object_t *obj, const char *var,
     values[n] = xstrdup ("");
     names[n] = xstrdup (_("Auto"));
     n++;
-#if (_WIN32_WINNT < 0x0601)
+#if (_WIN32_WINNT < _WIN32_WINNT_WIN7)
     if (version >= 0x0601)
 #endif
     {
@@ -946,7 +949,7 @@ size_t EnumClockSource (vlc_object_t *obj, const char *var,
         names[n] = xstrdup ("Interrupt time");
         n++;
     }
-#if (_WIN32_WINNT < 0x0600)
+#if (_WIN32_WINNT < _WIN32_WINNT_VISTA)
     if (version >= 0x0600)
 #endif
     {
@@ -973,80 +976,6 @@ size_t EnumClockSource (vlc_object_t *obj, const char *var,
 }
 
 
-/*** Timers ***/
-struct vlc_timer
-{
-    HANDLE handle;
-    void (*func) (void *);
-    void *data;
-};
-
-static void CALLBACK vlc_timer_do (void *val, BOOLEAN timeout)
-{
-    struct vlc_timer *timer = val;
-
-    assert (timeout);
-    timer->func (timer->data);
-}
-
-int vlc_timer_create (vlc_timer_t *id, void (*func) (void *), void *data)
-{
-    struct vlc_timer *timer = malloc (sizeof (*timer));
-
-    if (timer == NULL)
-        return ENOMEM;
-    timer->func = func;
-    timer->data = data;
-    timer->handle = INVALID_HANDLE_VALUE;
-    *id = timer;
-    return 0;
-}
-
-void vlc_timer_destroy (vlc_timer_t timer)
-{
-#if !VLC_WINSTORE_APP
-    if (timer->handle != INVALID_HANDLE_VALUE)
-        DeleteTimerQueueTimer (NULL, timer->handle, INVALID_HANDLE_VALUE);
-#endif
-    free (timer);
-}
-
-void vlc_timer_schedule (vlc_timer_t timer, bool absolute,
-                         mtime_t value, mtime_t interval)
-{
-    if (timer->handle != INVALID_HANDLE_VALUE)
-    {
-#if !VLC_WINSTORE_APP
-        DeleteTimerQueueTimer (NULL, timer->handle, INVALID_HANDLE_VALUE);
-#endif
-        timer->handle = INVALID_HANDLE_VALUE;
-    }
-    if (value == 0)
-        return; /* Disarm */
-
-    if (absolute)
-    {
-        value -= mdate ();
-        if (value < 0)
-            value = 0;
-    }
-    value = (value + 999) / 1000;
-    interval = (interval + 999) / 1000;
-
-#if !VLC_WINSTORE_APP
-    if (!CreateTimerQueueTimer (&timer->handle, NULL, vlc_timer_do, timer,
-                                value, interval, WT_EXECUTEDEFAULT))
-#endif
-        abort ();
-}
-
-unsigned vlc_timer_getoverrun (vlc_timer_t timer)
-{
-    (void)timer;
-    return 0;
-}
-
-
 /*** CPU ***/
 unsigned vlc_GetCPUCount (void)
 {
@@ -1059,9 +988,36 @@ unsigned vlc_GetCPUCount (void)
 
 
 /*** Initialization ***/
-void vlc_threads_setup (libvlc_int_t *p_libvlc)
+static CRITICAL_SECTION setup_lock; /* FIXME: use INIT_ONCE */
+
+void vlc_threads_setup(libvlc_int_t *vlc)
 {
-    SelectClockSource (VLC_OBJECT(p_libvlc));
+    EnterCriticalSection(&setup_lock);
+    if (mdate_selected != mdate_default)
+    {
+        LeaveCriticalSection(&setup_lock);
+        return;
+    }
+
+    if (!SelectClockSource((vlc != NULL) ? VLC_OBJECT(vlc) : NULL))
+        abort();
+    assert(mdate_selected != mdate_default);
+
+#if !VLC_WINSTORE_APP
+    /* Raise default priority of the current process */
+#ifndef ABOVE_NORMAL_PRIORITY_CLASS
+#   define ABOVE_NORMAL_PRIORITY_CLASS 0x00008000
+#endif
+    if (var_InheritBool(vlc, "high-priority"))
+    {
+        if (SetPriorityClass(GetCurrentProcess(), ABOVE_NORMAL_PRIORITY_CLASS)
+         || SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS))
+            msg_Dbg(vlc, "raised process priority");
+        else
+            msg_Dbg(vlc, "could not raise process priority");
+    }
+#endif
+    LeaveCriticalSection(&setup_lock);
 }
 
 #define LOOKUP(s) (((s##_) = (void *)GetProcAddress(h, #s)) != NULL)
@@ -1105,9 +1061,9 @@ BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved)
             thread_key = TlsAlloc();
             if (unlikely(thread_key == TLS_OUT_OF_INDEXES))
                 return FALSE;
-            InitializeCriticalSection (&clock_lock);
-            vlc_mutex_init (&super_mutex);
-            vlc_cond_init (&super_variable);
+            InitializeCriticalSection(&setup_lock);
+            InitializeCriticalSection(&super_mutex);
+            InitializeConditionVariable(&super_variable);
             vlc_rwlock_init (&config_lock);
             vlc_CPU_init ();
             break;
@@ -1115,9 +1071,8 @@ BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved)
 
         case DLL_PROCESS_DETACH:
             vlc_rwlock_destroy (&config_lock);
-            vlc_cond_destroy (&super_variable);
-            vlc_mutex_destroy (&super_mutex);
-            DeleteCriticalSection (&clock_lock);
+            DeleteCriticalSection(&super_mutex);
+            DeleteCriticalSection(&setup_lock);
             TlsFree(thread_key);
 #if (_WIN32_WINNT < _WIN32_WINNT_WIN8)
             if (WaitOnAddress_ == WaitOnAddressFallback)
